#include "historywindow.h"



#define HWT_MAKECHECKBOX(wid, txt, ident)													\
	wid = gtk_check_button_new_with_mnemonic(txt);											\
	g_object_set_data(G_OBJECT(wid), "icqnd-hdata", (gpointer)ident);						\
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid), 									\
								 (((IMHistoryManager*)manager)->readMask & ident)>0);		\
	g_signal_connect(wid, "clicked", G_CALLBACK(this->cb_maskCheckButtonClicked), this);	



historyWindow::historyWindow()
{
	wType = UE_HISTORY;
	hoveredLink = NULL;
}



historyWindow::~historyWindow()
{
	delete manager;
}



GtkWidget* historyWindow::createWindowContent()
{
	GtkWidget *vbox1,
			  *frame1,
			  *frame2,
			  *frame3,
			  *expander1,
			  *table1,
			  *label1,
			  *hbox1,
			  *hbox2,
			  *scrolledwindow1;

	GtkWidget *reverseCheckbox,	
			  *messageCheckbox,
			  *smsCheckbox,
			  *fileCheckbox,
			  *chatCheckbox,
			  *contactsCheckbox,
			  *linksCheckbox,
			  *incomingCheckbox,
			  *outgoingCheckbox,
			  *advancedFilterButton;
			  
	GtkTextBuffer *tb;
	gchar		*titleText,
				*fstr;
	

	// get neccessary properties
	settings_getSettings()->getProperties("conversations",
										  "showSmileys", &showSmileys,
										  "showURIs", &showURIs,
										  NULL);

	
	
	// create headline (progress bar, back and forward button, reverse label)
	historyPosition = gtk_progress_bar_new ();
	
	backButton = u_createStockImageButton(GTK_STOCK_GO_BACK);
	g_signal_connect_swapped(backButton, "clicked", G_CALLBACK(this->cb_backButtonClicked), this);
	forwardButton = u_createStockImageButton(GTK_STOCK_GO_FORWARD);
	g_signal_connect_swapped(forwardButton, "clicked", G_CALLBACK(this->cb_forwardButtonClicked), this);
	
	reverseCheckbox = gtk_check_button_new_with_mnemonic(tr("_Reverse"));
	g_signal_connect(reverseCheckbox, "clicked", G_CALLBACK(this->cb_reverseButtonClicked), this);
	
	hbox1 = gtk_hbox_new (FALSE, 3);
	gtk_box_pack_start (GTK_BOX (hbox1), historyPosition, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (hbox1), backButton, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (hbox1), forwardButton, FALSE, FALSE, 0);	
	gtk_box_pack_start (GTK_BOX (hbox1), reverseCheckbox, FALSE, FALSE, 0);

	frame1 = gtk_frame_new (NULL);
	gtk_frame_set_shadow_type (GTK_FRAME (frame1), GTK_SHADOW_OUT);
	gtk_container_add (GTK_CONTAINER (frame1), hbox1);	



	
	// create the text view	
	scrolledwindow1 = u_createTextView(&historyView);
	gtk_text_view_set_editable(GTK_TEXT_VIEW(historyView), FALSE);
	tb = gtk_text_buffer_new(settings_getSettings()->textTags);
	gtk_text_view_set_buffer(GTK_TEXT_VIEW(historyView), tb);
	g_object_unref(tb);
	g_signal_connect(historyView, "motion-notify-event", G_CALLBACK(this->cb_historyViewMouseMoved), this);
	g_signal_connect(historyView, "visibility-notify-event", G_CALLBACK(this->cb_historyViewVisibilityChanged), this);
	g_signal_connect_swapped(historyView, "button-press-event", G_CALLBACK(this->cb_historyViewURLPressed), this);



	// create the filter tab
	filterEntry = gtk_entry_new ();
	g_signal_connect_swapped(filterEntry, "activate", G_CALLBACK(this->cb_filterButtonClicked), this);

	filterButton 		 = u_createTextStockImageButton(tr("_Filter"), GTK_STOCK_FIND);
	g_signal_connect_swapped(filterButton, "clicked", G_CALLBACK(this->cb_filterButtonClicked), this);
	advancedFilterButton = u_createTextStockImageButton(tr("More..."), GTK_STOCK_FIND_AND_REPLACE);
	g_signal_connect_swapped(advancedFilterButton, "clicked", G_CALLBACK(this->cb_advancedFilterButtonClicked), this);
	
	hbox1 = gtk_hbox_new (FALSE, 3);
	gtk_box_pack_start(GTK_BOX(hbox1), filterEntry, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox1), filterButton, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox1), advancedFilterButton, FALSE, FALSE, 0);
	
	

	// create the expander and stuff
	table1 = gtk_table_new (2, 3, FALSE);
	
	HWT_MAKECHECKBOX(messageCheckbox, tr("Messages"), HRT_MESSAGE);
	gtk_table_attach (GTK_TABLE (table1), messageCheckbox, 0, 1, 0, 1,
					(GtkAttachOptions) (GTK_FILL),
					(GtkAttachOptions) (0), 0, 0);
	
	HWT_MAKECHECKBOX(smsCheckbox, tr("SMS"), HRT_SMS);
	
	gtk_table_attach (GTK_TABLE (table1), smsCheckbox, 2, 3, 0, 1,
					(GtkAttachOptions) (GTK_FILL),
					(GtkAttachOptions) (0), 0, 0);
	
	HWT_MAKECHECKBOX(fileCheckbox, tr("File transfers"), HRT_FILE);
	
	gtk_table_attach (GTK_TABLE (table1), fileCheckbox, 0, 1, 1, 2,
					(GtkAttachOptions) (GTK_FILL),
					(GtkAttachOptions) (0), 0, 0);
	
	HWT_MAKECHECKBOX(chatCheckbox, tr("Chat events"), HRT_CHAT);
	
	gtk_table_attach (GTK_TABLE (table1), chatCheckbox, 1, 2, 1, 2,
					(GtkAttachOptions) (GTK_FILL),
					(GtkAttachOptions) (0), 0, 0);

	HWT_MAKECHECKBOX(contactsCheckbox, tr("Contacts"), HRT_CONTACT);
	
	gtk_table_attach (GTK_TABLE (table1), contactsCheckbox, 2, 3, 1, 2,
					(GtkAttachOptions) (GTK_FILL),
					(GtkAttachOptions) (0), 0, 0);

	HWT_MAKECHECKBOX(linksCheckbox, tr("Links"), HRT_URL);
	
	gtk_table_attach (GTK_TABLE (table1), linksCheckbox, 1, 2, 0, 1,
					(GtkAttachOptions) (GTK_FILL),
					(GtkAttachOptions) (0), 0, 0);

	frame2 = gtk_frame_new(NULL);
	
	fstr = g_strdup_printf("<b>%s</b>", tr("Events to show"));
	label1 = gtk_label_new(fstr);
	g_free(fstr);
	gtk_label_set_use_markup(GTK_LABEL(label1), TRUE);
	gtk_frame_set_label_widget(GTK_FRAME(frame2), label1);
	gtk_container_add(GTK_CONTAINER(frame2), table1);
	
	


	HWT_MAKECHECKBOX(incomingCheckbox, tr("r_eceived events"), HRT_INCOMING);
	HWT_MAKECHECKBOX(outgoingCheckbox, tr("_sent events"), HRT_OUTGOING);

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (vbox1), incomingCheckbox, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (vbox1), outgoingCheckbox, FALSE, FALSE, 0);
	
	
	frame3 = gtk_frame_new (NULL);
	gtk_frame_set_shadow_type (GTK_FRAME (frame3), GTK_SHADOW_NONE);
		
	fstr = g_strdup_printf("<b>%s</b>", tr("direction"));
	label1 = gtk_label_new(fstr);
	g_free(fstr);
	gtk_label_set_use_markup (GTK_LABEL (label1), TRUE);	
	gtk_frame_set_label_widget (GTK_FRAME (frame3), label1);
	gtk_container_add(GTK_CONTAINER(frame3), vbox1);


	hbox2 = gtk_hbox_new (FALSE, 4);
	gtk_box_pack_start(GTK_BOX(hbox2), frame2, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox2), frame3, TRUE, TRUE, 0);
	
		
	expander1 = gtk_expander_new (NULL);
	gtk_expander_set_label_widget(GTK_EXPANDER(expander1), gtk_label_new(tr("More options")));	
	gtk_container_add(GTK_CONTAINER(expander1), hbox2);
		
	


	vbox1 = gtk_vbox_new (FALSE, 2);
	gtk_container_set_border_width (GTK_CONTAINER (vbox1), 5);
	
	gtk_box_pack_start(GTK_BOX(vbox1), frame1, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox1), scrolledwindow1, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox1), expander1, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox1), createButtonbar(), FALSE, TRUE, 0);
	
	setWindowSize(450, 400);
	titleText = g_strdup_printf(tr("History of %s"), manager->info->alias);
	setWindowTitle(titleText);
	g_free(titleText);
	
	if (!((IMHistoryManager*)manager)->loadHistory())
		u_showAlertMessage(tr("reading error!"), tr("The history couldn't be read!"), GTK_STOCK_DIALOG_ERROR);
	showEntries();
	
	addSettingsListener("conversations");

	return vbox1;
}



GtkWidget* historyWindow::createButtonbar()
{
	GtkWidget *hbox,
			  *refreshButton,
			  *closeButton;
	
	saveButton = gtk_button_new_from_stock (GTK_STOCK_SAVE);
	refreshButton = gtk_button_new_from_stock (GTK_STOCK_REFRESH);
	closeButton = gtk_button_new_from_stock (GTK_STOCK_CLOSE);

	
	hbox = gtk_hbox_new (FALSE, 5);
	
	gtk_box_pack_start (GTK_BOX (hbox), saveButton, FALSE, FALSE, 0);
	g_signal_connect_swapped(saveButton, "clicked", G_CALLBACK(this->cb_saveHistoryButtonClicked), this);
	gtk_box_pack_start (GTK_BOX (hbox), refreshButton, FALSE, FALSE, 0);
	g_signal_connect_swapped(refreshButton, "clicked", G_CALLBACK(this->cb_refreshButtonClicked), this);
	gtk_box_pack_end (GTK_BOX (hbox), closeButton, FALSE, FALSE, 0);	
	g_signal_connect_swapped(closeButton, "clicked", G_CALLBACK(this->cb_closeButtonClicked), this);

	return hbox;
}







void historyWindow::destroyWindowContent()
{
	((IMHistoryManager*)manager)->stopLoadingHistoryFiltered();

}



void historyWindow::updateProgressBar(gint rstart, gint rend, gint rsize)
{
	gchar	*tmp;
	gdouble perc;
	
	if (!rsize)
	{
		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(historyPosition), tr("No entries"));
		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(historyPosition), 0.0f);
		return;
		
	}
	
	tmp = g_strdup_printf(tr("Entries from %d to %d of %d"), rstart + 1, rend, rsize);
	
	if (rsize != 0)
		perc = (float)rend / (float)rsize;
	else
		perc = 1.0f;
	
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(historyPosition), tmp);
	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(historyPosition), perc);
	
	g_free(tmp);
}



void historyWindow::showEntries()
{	
	GList *entries;
	IMHistoryEntry* hentry;
	gint rstart = 0, rend = 0, rsize;
	gboolean doReverse;
	GtkTextBuffer* buf;
	GtkTextIter startIter, endIter;
	gchar	htime[100];
	gchar	*startText, *tag;	
	
	rsize = ((IMHistoryManager*)manager)->historyLength();
	doReverse = ((IMHistoryManager*)manager)->getReadReverse();		

	if (!doReverse)
		rstart = ((IMHistoryManager*)manager)->historyPosition();
	else
	{
		rend = ((IMHistoryManager*)manager)->historyPosition();
		if (rend<40 && rsize>40)
			rend = 40;
	}
	
	// get the history entries
	entries = ((IMHistoryManager*)manager)->getHistoryEntries(40, TRUE);
	
	
	if (!doReverse)
		rend = ((IMHistoryManager*)manager)->historyPosition();
	else
		rstart = ((IMHistoryManager*)manager)->historyPosition();
	
	
	// check whether the buttons have to be grayed out
	if (rstart == 0)
		gtk_widget_set_sensitive(backButton, FALSE);
	else
		gtk_widget_set_sensitive(backButton, TRUE);
	
	if (rend == rsize)
		gtk_widget_set_sensitive(forwardButton, FALSE);
	else
		gtk_widget_set_sensitive(forwardButton, TRUE);
	
	updateProgressBar(rstart, rend, rsize);
	
	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(historyView));	
	gtk_text_buffer_get_bounds(buf, &startIter, &endIter);
	gtk_text_buffer_delete(buf, &startIter, &endIter);
	
	if (!rsize)
		gtk_widget_set_sensitive(saveButton, FALSE);
	else
		gtk_widget_set_sensitive(saveButton, TRUE);
		
	while(entries)
	{
		hentry = (IMHistoryEntry*)entries->data;
		
		strftime(htime, 26, tr(" in %d.%m.%y at %H:%M:%S"), localtime(&hentry->time));
		
		if (hentry->incoming)
		{
			tag 	  = "incoming";
			startText = tr_NOOP("Received");
		} else
		{
			tag 	  = "outgoing";
			startText = tr_NOOP("Sent");
		}
		
		
		gtk_text_buffer_get_end_iter(buf, &endIter);			
		// add entry to a specific field
		gtk_text_buffer_insert_with_tags_by_name(buf, &endIter, tr(startText), -1, tag, "HL", NULL);
		gtk_text_buffer_insert_with_tags_by_name(buf, &endIter, htime, -1, tag, "HL", NULL);
		gtk_text_buffer_insert_with_tags_by_name(buf, &endIter, "\n\n", -1, "newline", NULL);
		u_insertTextInterpretedIntoView(hentry->text, "margin", FALSE, showURIs, showSmileys, &endIter, GTK_TEXT_VIEW(historyView));
		
		// gtk_text_buffer_insert_with_tags_by_name(buf, &endIter, hentry->text, -1, tag, "margin", NULL);
		gtk_text_buffer_insert_with_tags_by_name(buf, &endIter, "\n\n\n", -1, tag, NULL);		

		
		entries = entries->next;
	}
}




void historyWindow::startFiltering()
{
	switch( ((IMHistoryManager*)manager)->loadHistoryFiltered((gchar*)gtk_entry_get_text(GTK_ENTRY(filterEntry))))
	{
		// everything went okay
		case 0:	
			break;
		
		// history is empty or could not be loaded
		case 1:
		case 2:
			break;
		
		// the pattern was empty -> history was restored and resetted
		case 3:
			showEntries();
			break;
		
		// the pattern is invalid
		case 4:
			u_showAlertMessage(tr("input error"), tr("The pattern is too short.\nPlease enter at lease 3 characters!"), GTK_STOCK_DIALOG_WARNING);
			break;
	}
}

void historyWindow::settingsChangedCallback(gchar* sname, GList *values)
{	
	settingsProperty	*prop;
	gboolean			refreshView;
	
	refreshView = FALSE;
	
	if (!strcmp(sname, "conversations"))
		for (; values; values = values->next)
		{
			prop = (settingsProperty*)values->data;
			
			if (!strcmp(prop->name, "showSmileys"))
			{
				showSmileys = g_value_get_boolean(prop->val);
				refreshView = TRUE;
			} else
			if (!strcmp(prop->name, "showURIs"))
			{
				showURIs = g_value_get_boolean(prop->val);
				refreshView = TRUE;
			}
		}
		
	if (refreshView)
	{
		if (((IMHistoryManager*)manager)->getReadReverse())
			((IMHistoryManager*)manager)->moveHistoryPosition(40);
		else
			((IMHistoryManager*)manager)->moveHistoryPosition(-40);
		
		showEntries();
	}
}





gboolean historyWindow::eventCallback(gint command, gint lastResult, gpointer info)
{
	if (command != EC_LOADHISTORY)
		return TRUE;
	
	switch(lastResult)
	{
		// filtering started
		case 0:
			gtk_progress_bar_set_text(GTK_PROGRESS_BAR(historyPosition), tr("Searching ..."));	
			break;
		
		// filtering finished
		case 100:
			showEntries();
			break;
		
		// filtering is running
		default:
			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(historyPosition), (float)lastResult/100.0f);
			break;
	}
		
	return TRUE;
}



void historyWindow::cb_filterButtonClicked(historyWindow* hw)
{
	hw->startFiltering();
}




void historyWindow::cb_advancedFilterButtonClicked(historyWindow* hw)
{
	u_showAlertMessage("Be patient ...", "Work is in progress", GTK_STOCK_PREFERENCES);
}




void historyWindow::cb_saveHistoryButtonClicked(historyWindow* hw)
{
	GtkWidget	*fileSelection;
	gboolean   	saveFinished,
			   	overwriteFile;
	gchar		*fileName;
	
	fileSelection = gtk_file_chooser_dialog_new(tr("Select a name for the history file"),
												GTK_WINDOW(hw->window),
												GTK_FILE_CHOOSER_ACTION_SAVE,
					      					 	GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				      					 		GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
												NULL
												);
	gtk_widget_show_all(fileSelection);	
	
	saveFinished = TRUE;
	overwriteFile = FALSE;

	
	do
	{
		// FIXME: this is somehow dirty
		if (overwriteFile || gtk_dialog_run(GTK_DIALOG(fileSelection)) == GTK_RESPONSE_ACCEPT)
		{
			fileName = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(fileSelection));
			
			switch(((IMHistoryManager*)hw->manager)->saveHistory(fileName,overwriteFile))
			{
				// okay
				case 0:
					u_showAlertMessage(tr("History written!"), tr("The history was successfully written!"), GTK_STOCK_DIALOG_INFO);
					saveFinished = TRUE;
					break;
			
				// file exists
				case 1:
					switch (u_showYesNoDialog(tr("File already exists"), tr("The specified file already exists.\nShall it be overwritten?"), GTK_STOCK_DIALOG_QUESTION, TRUE))
					{
						case GTK_RESPONSE_NO:
							saveFinished = FALSE;
							break;
						
						case GTK_RESPONSE_CANCEL:
							saveFinished = TRUE;
							break;
						
						case GTK_RESPONSE_YES:
							saveFinished = FALSE;
							overwriteFile = TRUE;
							break;
					}
					break;
					
				// file name is a directory
				case 2:
					u_showAlertMessage(tr("Write error"), tr("There is a directory with the same name\nas the one specified. Please choose\nanother name!"), GTK_STOCK_DIALOG_WARNING);
					overwriteFile = FALSE;
					saveFinished = FALSE;
					break;
				
				// file couldn't be created
				case 3:
					u_showAlertMessage(tr("Write error"), tr("The file couldn't be created!\nPlease check the file name\nand your permissions on the directory\nthe file is in!"), GTK_STOCK_DIALOG_ERROR);
					saveFinished = TRUE;
					break;
				
				// file couldn't be written
				case 4:
					u_showAlertMessage(tr("Write error"), tr("The file could be created but not filled!\nPlease check your permissions!"), GTK_STOCK_DIALOG_ERROR);
					saveFinished = TRUE;
					break;
			}
			
			g_free(fileName);
		}
	} while (!saveFinished);
	

	gtk_widget_destroy(fileSelection);
}




void historyWindow::cb_refreshButtonClicked(historyWindow* hw)
{
	IMHistoryManager* hman = (IMHistoryManager*)hw->manager;
	
	if (hman->pattern)
		hman->loadHistoryFiltered(hman->pattern);
	else
	{
		hman->loadHistory();
		hw->showEntries();
	}
	
	
}




void historyWindow::cb_closeButtonClicked(historyWindow* hw)
{
	hw->destroyWindow();
	delete hw;
}




void historyWindow::cb_reverseButtonClicked(GtkWidget* w, historyWindow* hw)
{
	gboolean reverseState;
	
	reverseState = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
	
	((IMHistoryManager*)hw->manager)->setReadReverse(reverseState);
	((IMHistoryManager*)hw->manager)->resetHistoryReading();
	hw->showEntries();
}



void historyWindow::cb_forwardButtonClicked(historyWindow* hw)
{
	if (((IMHistoryManager*)hw->manager)->getReadReverse())
		((IMHistoryManager*)hw->manager)->moveHistoryPosition(80);
	hw->showEntries();
}



void historyWindow::cb_backButtonClicked(historyWindow* hw)
{
	if (!((IMHistoryManager*)hw->manager)->getReadReverse())
		((IMHistoryManager*)hw->manager)->moveHistoryPosition(-80);
	hw->showEntries();
}




void historyWindow::cb_maskCheckButtonClicked(GtkWidget* w, historyWindow* hw)
{
	gpointer data;
	
	data = g_object_get_data(G_OBJECT(w), "icqnd-hdata");
	
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)))
		((IMHistoryManager*)hw->manager)->readMask |= GPOINTER_TO_UINT(data);
	else
	{
		if (((IMHistoryManager*)hw->manager)->readMask & GPOINTER_TO_UINT(data))
			((IMHistoryManager*)hw->manager)->readMask ^= GPOINTER_TO_UINT(data);
	}
	((IMHistoryManager*)hw->manager)->resetHistoryReading();
	hw->showEntries();	
}


gboolean historyWindow::cb_historyViewMouseMoved(GtkWidget* v, GdkEventMotion* event, historyWindow *self)
{
	u_checkInterpretedViewForURLCursor((gint)event->x, (gint)event->y, GTK_TEXT_VIEW(self->historyView), &self->hoveredLink);

	gdk_window_get_pointer(self->historyView->window, NULL, NULL, NULL);

	return FALSE;
}


gboolean historyWindow::cb_historyViewVisibilityChanged(GtkWidget* v, GdkEventVisibility* event, historyWindow *self)
{
	gint cX, cY;
		
 	gdk_window_get_pointer (self->historyView->window, &cX, &cY, NULL);
	u_checkInterpretedViewForURLCursor(cX, cY, GTK_TEXT_VIEW(self->historyView), &self->hoveredLink);
	
	return FALSE;
}


gboolean historyWindow::cb_historyViewURLPressed(historyWindow* self, GdkEventButton* event)
{	
	if (self->hoveredLink && event->button == 1)
		uu_openURL(self->hoveredLink);
	return FALSE;
}
