/***************************************************************************
 *            imbuddy.cpp
 *
 *  Mon Jan 24 22:04:43 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#include "imbuddy.h"

#ifdef USE_SCRNSAVER
	#include <X11/Xlib.h>
	#include <X11/Xutil.h>
	#include <X11/extensions/scrnsaver.h>
	
XScreenSaverInfo* xIdleInfo = NULL;
#endif

#ifdef HAVE_LIBGPGME
	#include "gpgmanager.h"
#endif

gboolean ib_highlightOnline = FALSE,
		 ib_highlightARChecked = FALSE,
		 ib_autoOfflineEnabled = FALSE,
		 ib_autoAwayEnabled = FALSE,
		 ib_autoNAEnabled = FALSE;
gulong	 ib_autoOffline = 0,
		 ib_autoAway = 0,
		 ib_autoNA = 0;



IMBuddy::IMBuddy(IMUserDaemon *_user)
{
	if (_user)
	{
		// copy user reference and set callback function
		user = _user;
		user->dowry = this;
		user->addCallback((IMEventCallback)this->cb_buddyCallback, this);
	} else
		user = NULL;
		
	cEntries   = NULL;	
	openEvents = NULL;
	ARMan 	   = NULL;
	
	if (user)
		settings_getSettings()->installListener("appearance", (settingChangedFunc)this->cb_settingsChangedCallback, this, FALSE);
}


IMBuddy::~IMBuddy()
{
	closeAllEvents();
	
	if (ARMan)
		delete ARMan;
	
	if (user)
		delete user;
	
	settings_getSettings()->removeListenerByData(this);
}



basicWindow* IMBuddy::getOpenedEvent(userEventType evType)
{
	GList *search;

	
	search = openEvents;
	while(search)
	{
		if (((basicWindow*)search->data)->wType == evType)
		{
			return (basicWindow*)search->data;
		}
		search = search->next;
	}	
	
	return NULL;
}


	
gboolean IMBuddy::openEvent(userEventType evType, gboolean isIncoming)
{
	basicWindow	   *newEvent;
	IMEventManager *newMan;
	gboolean	   openWindow,
				   setConvURL;
	gchar			*text,
					*tmpText;
	generalEventInfo	*evInfo;
	
	// FIXME: this is somehow dirty
	if (evType == UE_URL)
	{
		evType = UE_MESSAGE;
		setConvURL = TRUE;
	} else
		setConvURL = FALSE;
	
	newEvent = NULL;
	
	if ( (newEvent = getOpenedEvent(evType)))
		newEvent->focusWindow();

	
	
	if (!newEvent)
	{
		openWindow = TRUE;
	
		switch(evType)
		{
			case UE_MESSAGE:
				newEvent = (basicWindow*)new conversationWindow();
				newMan = (IMEventManager*)new IMMessageManager(user->info);			
				break;
		
			case UE_FILETRANSFER:
				newEvent = (basicWindow*)new fileTransferWindow(FALSE);
				newMan = (IMEventManager*)new IMFileTransferManager(user->info);
				openWindow = FALSE;
				break;

			case UE_FILETRANSFERREQUEST:
				newEvent = (basicWindow*)new fileTransferWindow(TRUE);
				newMan = (IMEventManager*)new IMFileTransferManager(user->info);
				newEvent->manager = newMan;		
				((fileTransferWindow*)newEvent)->createRequestDialog(NULL);		
				openWindow = FALSE;
				break;
			
			case UE_CONTACTS:
			case UE_CONTACTSREQUEST:
				newEvent = new contactsWindow(evType == UE_CONTACTSREQUEST);
				newMan = new IMContactsManager(user->info);
				break;


			case UE_CHATREQUEST:
				newEvent = (basicWindow*)new chatWindow(TRUE);
				newMan = (IMEventManager*)new IMChatManager(user->info);
				newEvent->manager = newMan;	
				((chatWindow*)newEvent)->createRequestDialog(NULL);
				openWindow = FALSE;
				break;
		
		
			case UE_CHAT:
				newEvent = (basicWindow*)new chatWindow(FALSE);
				newMan = (IMEventManager*)new IMChatManager(user->info);
				openWindow = FALSE;
				break;
		
		
			case UE_USERINFO:
				newEvent = (basicWindow*)new userInfoWindow();
				newMan = new IMUserUpdateManager(user->info);
				break;
		
		
			case UE_HISTORY:			
				newEvent = (basicWindow*)new historyWindow();	
				newMan = new IMHistoryManager(user->info);
				break;

			case UE_AUTHORIZATION:
			case UE_AUTHORIZATIONREQUEST:
			case UE_AUTHORIZATIONGRANT:
				if (isIncoming && evType == UE_AUTHORIZATIONGRANT)
				{
					newEvent = (basicWindow*)new simpleMessageWindow(UE_AUTHORIZATIONGRANT, tr("The user accepted your request to add him to his contact list."), tr("%s authorized you"));
					newMan = new IMAuthorizationManager(user->info);
				} else
				{
					newEvent = (basicWindow*)new authorizeWindow(evType);
					newMan = new IMAuthorizationManager(user->info);
				}
				break;		
		
			case UE_AUTHORIZATIONREFUSE:
				newEvent = (basicWindow*)new simpleMessageWindow(UE_AUTHORIZATIONREFUSE, tr("The user rejected your request to add him to your contact list"), tr("%s rejected your request"));
				newMan = new IMAuthorizationManager(user->info);
				break;
		
			case UE_AUTORESPONSESET:
				newEvent = (basicWindow*)new autoResponseWindow(0);
				newMan = new IMAutoResponseManager(user->info, user->owner == NULL);
				break;
		
		
			case UE_AUTORESPONSEGET:
				newEvent = (basicWindow*)new fetchResponseWindow();
				newMan = new IMAutoResponseManager(user->info, FALSE);
				break;
		
			case UE_ENCRYPTION:
				newEvent = (basicWindow*)new securityWindow();
				newMan = new IMSecurityManager(user->info);
				break;
						
			case UE_ADDEDTOLIST:
				newEvent = (basicWindow*)new simpleMessageWindow(UE_ADDEDTOLIST, tr("The user has added you to his contact list."), tr("%s added you"));
				newMan = new IMMiscManager(user->info);
				break;
			
			case UE_EDITUSERMODES:
				newEvent = (basicWindow*)new modesWindow();
				newMan = new IMEventManager(user->info);
				break;
			
		
			case UE_WEBPANEL:
			case UE_PAGER:
			case UE_SERVERMSG:
				newMan = new IMNotificationManager(user->info);
				user->addManager(newMan);
				user->initializeManagerForUser(newMan);
				// We now have the last event
				evInfo = (generalEventInfo*)newMan->getEvent(EO_LAST | EO_SYSTEM | EO_NORMAL);
				tmpText = g_markup_escape_text(evInfo->text, -1);
				newMan->deleteEvent(EO_LAST | EO_SYSTEM | EO_NORMAL);
				delete newMan;
			
				text = NULL;
			
				switch(evType)
				{
					case UE_WEBPANEL:
						text = g_strdup_printf("<b>%s</b>\n\n%s", tr("You have received a message from your web panel."), tmpText);
						break;
					
					case UE_PAGER:
						text = g_strdup_printf("<b>%s</b>\n\n%s", tr("You have received a message from the pager."), tmpText);
						break;
					
					case UE_SERVERMSG:
						text = g_strdup_printf("<b>%s</b>\n\n%s", tr("You have received a server message."), tmpText);
						break;
					
					default: break;
				}
				
				u_showAlertMessage(tr("You have received a message"), text, GTK_STOCK_DIALOG_INFO);
				g_free(text);
				g_free(tmpText);
				break;				

			default:
				fprintf(stderr, "IMBuddy::openEvent: unknown new action!\n");
				return FALSE;
		}
	
		if (newEvent)
		{
			// set the newMan->setCallback((IMEventCallback)newEvent->cb_eventCallback, newEvent);callback and create the window if neccessary
			newEvent->manager = newMan;
			newEvent->setDestroyCallback((IMEventCallback)this->cb_buddyCallback, this);
			newMan->addCallback((IMEventCallback)newEvent->cb_eventCallback, newEvent);
			user->addManager(newMan);
			if (openWindow)
				newEvent->createWindow();
			user->initializeManagerForUser(newMan);
	
			openEvents = g_list_append(openEvents, newEvent);
		}
	};
	
	
	// post processing
	switch(evType)
	{
		case UE_MESSAGE:
			if (isIncoming)
				break;
			if (setConvURL)
				((conversationWindow*)newEvent)->enableURLMode();
			else			
				((conversationWindow*)newEvent)->disableURLMode();
			break;
			
		default:
			break;
	}
	
	return TRUE;
}



gboolean IMBuddy::closeEvent(userEventType evType)
{
	return TRUE;	
}


void IMBuddy::closeAllEvents()
{
	GList	*eventsSearch,
			*search;
	
	if (!openEvents)
		return;
	
	eventsSearch = g_list_copy(openEvents);
	
	for (search = eventsSearch; search; search = search->next)
	{
		((basicWindow*)eventsSearch->data)->destroyWindow();
		delete (basicWindow*)eventsSearch->data;		
	}

	g_list_free(eventsSearch);
	g_list_free(openEvents);
	
	openEvents = NULL;	
}





void IMBuddy::updateContactListStatus(gboolean updateAlias)
{
	GList			 *search;
	contactListUser  *entry;
		
	search = cEntries;
	while(search)
	{
		entry = (contactListUser*)search->data;		

		entry->setStatus(user->info->status, 
						 i_getIcons()->getUserStatusPixbuf(user));
		entry->updateColors();	
		
		if (updateAlias)
			entry->updateEntry(FALSE);
		
		search = search->next;
	}
	
	
}


void IMBuddy::updateContactListEvents(basicEventInfo* ev, gboolean moveToTop)
{
	GdkPixbuf	*toFlash;
	GList		*search;
	
	toFlash = getEventSymbol(ev);		
		
	search = cEntries;
	while(search)
	{
		((contactListUser*)search->data)->blinkIcon(toFlash);
		((contactListUser*)search->data)->updateEntry(FALSE);
					
		search = search->next;
	}	
}


void IMBuddy::updateContactListStates()
{
	gulong	newState;
	GList	*search;
	
	newState = 0;
	if (user->info->specialAR && *user->info->specialAR)
		newState|=1;
	if (user->info->birthday.hasBirthday)
		newState|=2;
	if (user->info->home.mobile && *user->info->home.mobile)
		newState|=4;
	if (user->info->home.phone && *user->info->home.phone)
		newState|=8;
	if (user->info->typing == 2)
		newState|=16;
	
	for (search = cEntries; search; search = search->next)
		((contactListUser*)search->data)->setStateMask(newState);
}



void IMBuddy::putUserIntoGroups(contactListEntry* root, gboolean useRealName)
{
	GList				*search;
	contactListEntry	*entry,
						*newEntry;

	
	search = user->groups;
	while(search)
	{
		entry = root->findChildByID(((groupInfo*)search->data)->icqndID, CE_GROUP);
				
		if (entry)
		{
			// doOpen = ((contactListGroup*)entry)->getGroupOpened();
			/*
			if (useRealName)
				nameToShow = uu_getRealNameOrAlias(user->info, TRUE);
			else
				nameToShow = g_strdup(user->info->formattedAlias);
			
			newEntry = new contactListUser(
							user->info->licqID,
							nameToShow,
							i_getIcons()->getUserStatusPixbuf(user),
			//				uu_getStatusSymbol(user->info->status, user->info->isInvisible),
							user->info->status,
							user,
							cEntries);
				
			g_free(nameToShow);			*/
			newEntry = new contactListUser(user, cEntries);
			((contactListUser*)newEntry)->setStatus(user->info->status, i_getIcons()->getUserStatusPixbuf(user));			
			((contactListUser*)newEntry)->updateColors();

			entry->addChild((contactListEntry*)newEntry);
				
			/*		
			group = (groupInfo*)entry->data;
			((contactListGroup*)entry)->setGroupOpened(doOpen);*/
			cEntries = g_list_append(cEntries, newEntry);
					
		} else
			fprintf(stderr, "IMOwner::putUserIntoGroups(): No group was found for %s!\n", user->info->alias); 				
								
		search = search->next;
	}	
	
	updateContactListStates();
}



GdkPixbuf* IMBuddy::getEventSymbol(basicEventInfo *info)
{
	gint last;


	if (!info)
	{
		info = user->getEvent(EO_LAST | EO_NORMAL | EO_SYSTEM);
		if (info)
			last = info->type;
		else
			last = -1;
	}
	else
		last = info->type;
	
	return i_getIcons()->getEventPixbuf(last);	
}





// =============================================================================

gboolean IMBuddy::cb_buddyCallback(userEventType type, 
					               gint command, 
								   gint lastResult, gpointer info, 
								   IMBuddy* self)
{
	GList		*search;
		
	switch(command)
	{
		// some window wants another window to be opened
		case EC_FROMMANAGER:
			self->openEvent((userEventType)lastResult, FALSE);
			break;
		
		// status was updated (FIXME: analyze this command further)
		case UC_UPDATEUSER:
			if (lastResult == USER_UPDATE_STATUS || lastResult == USER_UPDATE_ALL || lastResult == USER_UPDATE_GENERAL)
				self->updateContactListStatus(lastResult != USER_UPDATE_STATUS);
			
			self->updateContactListStates();
		
			// animate the user if he went online
			if (ib_highlightOnline && self->user->info->wentJustOnline)
				for (search = self->cEntries; search; search = search->next)
					((contactListUser*)search->data)->flashName(CF_LETTERS);
			break;
			
		case UC_AUTORESPONSE_CHECKED:
			if (ib_highlightARChecked)
				for (search = self->cEntries; search; search = search->next)
					((contactListUser*)search->data)->flashName(CF_STYLE);
			break;
		
		case UC_SETAUTORESPONSE:
			self->updateContactListStates();
			break;
				
					
		// a new event was received or finished
		case UC_EVENT_FINISHED:
		case UC_EVENT_RECEIVED:
			if (type != UE_AUTORESPONSEGET)
				self->updateContactListEvents((basicEventInfo*)info, command == UC_EVENT_RECEIVED);
			break;
					
			
		// a window was destroyed
		case WC_DESTROYED:
			search = self->openEvents;
			while(search)
			{
				if (search->data == info)
				{
					self->openEvents = g_list_remove(self->openEvents, search->data);
					//fprintf(stderr, "IMBuddy::cb_buddyCallback: event window destroyed (%d left)!\n", g_list_length(self->openEvents));
					break;
				}
				
				search = search->next;
			}
			break;
	}	
	
	return FALSE;
}



void IMBuddy::cb_settingsChangedCallback(gchar* sname, GList *values, IMBuddy* self)
{
	gboolean 			redrawWindows;
	settingsProperty	*prop;
	GList				*search;
	basicWindow			*wnd;
	
	redrawWindows = FALSE;
	
	while(values)
	{
		prop = (settingsProperty*)values->data;
		
		if (!strcmp(prop->name, "EventTheme"))
		{
			redrawWindows = TRUE;
			break;
		}
		
		values = values->next;
	}
	
	// redraw all relevant areas
	if (redrawWindows)
	{
		search = self->openEvents;
		while(search)
		{
			wnd = (basicWindow*)search->data;
			
			if (wnd->content)
				gtk_widget_queue_draw(wnd->content);
			
			search = search->next;
		}
	}
}




// ==================== IMOwner ================================================

IMOwner::IMOwner():
IMBuddy::IMBuddy(NULL)
{
	GList				*rawProtos,
						*groups,
						*search;
	IMBuddy				*newProto;
	IMGroupManager		*groupMan;
	gulong				openedGroups,
						i;
	gboolean			sendViaServer;
	
	setupDefaultSettings();
	
	// setup logging
	logDaemon = new IMLogDaemon();
	
	
	groupMan = new IMGroupManager();
	groups = groupMan->loadGroups();
	IO_setGroupManager(groupMan);	
	
	
	// set the opened groups and load other cl settings
	settings_getSettings()->getProperties("contactlist", 
										  "lastOpenedGroups", &openedGroups,
										  "highlightARChecked", &ib_highlightARChecked,
										  "highlightOnline", &ib_highlightOnline,
										  NULL);
													  
	settings_getSettings()->getProperties("autoresponses",
										  "autoAway", &ib_autoAway,
										  "autoNA", &ib_autoNA,
										  "autoOffline", &ib_autoOffline,
										  "autoAwayEnabled", &ib_autoAwayEnabled,
										  "autoNAEnabled", &ib_autoNAEnabled,
										  "autoOfflineEnabled", &ib_autoOfflineEnabled,
										  NULL);
										  
	settings_getSettings()->getProperties("conversations",
										  "alwaysViaServer", &sendViaServer,
										  NULL);

	i=0;
	groups = groupMan->groups;
	while(groups)
	{
		((groupInfo*)groups->data)->opened = (openedGroups & (1<<i))!=0;
		i++;
		groups = groups->next;
	}
	
	eventCount = 0;
	ownerEventCount = 0;
	
	// connect to the general signal source
	IO_getGeneralSource()->addCallback((IMEventCallback)this->cb_generalCallback, this);
	
	// load all plugins
	IO_getPluginsList();
	
	// get all protocol owners
	rawProtos = IO_getOwnerList();

	while(rawProtos)
	{
		newProto = new IMBuddy((IMUserDaemon*)rawProtos->data);
		newProto->user->addCallback((IMEventCallback)this->cb_ownerCallback, this);
		newProto->user->dowry = newProto;

		((IMOwnerDaemon*)newProto->user)->loadContacts();
		
		// Shall we send via server?
		if (sendViaServer)
			for (search = ((IMOwnerDaemon*)newProto->user)->users; search; search = search->next)
			{
				((IMUserDaemon*)search->data)->info->sendViaServerByUser = TRUE;
				((IMUserDaemon*)search->data)->info->sendViaServer = TRUE;
			}
				
		rawProtos = rawProtos->next;
	}
	
	// create main window
	window = new mainWindow((mwMenuClickFunction)this->cb_menuCallback, (mwDropFunction)this->cb_droppingCallback, G_CALLBACK(this->cb_quitWindowCallback), this);	
	
	window->setWindowTitle(FALSE);

	
	// check for saved owner events
	search = IO_getOwnerList();
	while(search)
	{
		((IMUserDaemon*)search->data)->dispatchPendingEvents();
		
		search = search->next;
	}
		
	if (g_list_length(IO_getOwnerList())>0)
	{
		rebuildContactList(TRUE);
		autoConnect();	
	}
	else
	{
		window->setLockUserMenus(TRUE);
		openEvent(UE_REGISTEROWNER, 0);
	} 		
	
	settings_getSettings()->installListener("appearance", (settingChangedFunc)this->cb_settingsChangedCallback, this, FALSE);	
	
	settings_getSettings()->installListener("autoresponses", (settingChangedFunc)this->cb_ownerSettingsChangedCallback, this, FALSE);
	settings_getSettings()->installListener("contactlist", (settingChangedFunc)this->cb_ownerSettingsChangedCallback, this, FALSE);
	settings_getSettings()->installListener("conversations", (settingChangedFunc)this->cb_ownerSettingsChangedCallback, this, FALSE);

	
	// install idle timeout	
#ifdef USE_SCRNSAVER
	fprintf(stderr, "IMOwner::IMOwner(): installing IDLE check timeout\n");
	g_timeout_add(10000, (GSourceFunc)this->cb_checkIDLETimeout, this);
#endif

	errorCode = 0;
}




gboolean IMOwner::setupDefaultSettings()
{
	settings	*config;
	GdkColor	log_errorCol,
				log_warningCol,
				log_packetCol,
				log_defaultCol,
				conv_messageCol,
				conv_typingCol,
				cl_selCol,
				cl_vlCol,
				cl_noCol,
				cl_idleCol,
				cl_wentOnlineCol,
				cl_groupCol;
	gchar		*browser;
	u_browserInfo	*browserInfo;
	
	// create the settings and set it as default
	config = new settings("licq_qnd-gui.conf");
	settings_setSettings(config);	
	
	// defaults for the logging window
	GDK_COLOR_SET(&log_errorCol, 0xFFFF, 0x0000, 0x0000);
	GDK_COLOR_SET(&log_warningCol, 0x8888, 0x0000, 0x0000);
	GDK_COLOR_SET(&log_packetCol, 0x4444, 0x4444, 0x4444);
	GDK_COLOR_SET(&log_defaultCol, 0x0000, 0x0000, 0x0000);
	
	config->installProperties("logwindow",
							  "errorColor", ICQND_TYPE_COLOR, &log_errorCol,
							  "warningColor", ICQND_TYPE_COLOR, &log_warningCol,
							  "packetColor", ICQND_TYPE_COLOR, &log_packetCol,
							  "defaultColor", ICQND_TYPE_COLOR, &log_defaultCol,
							  NULL);
	
	// defaults for the main window
	config->installProperties("mainwindow",
							  "width", G_TYPE_ULONG, ( gulong ) 100,
							  "height", G_TYPE_ULONG, ( gulong ) 300,
							  "positionX", G_TYPE_ULONG, ( gulong ) 0,
							  "positionY", G_TYPE_ULONG, ( gulong ) 0,
							  "showMainMenu", G_TYPE_BOOLEAN, TRUE,
							  "showNotifyBox", G_TYPE_BOOLEAN, TRUE,
							  "showStatusButtons", G_TYPE_BOOLEAN, TRUE,
							  "showWindowBorders", G_TYPE_BOOLEAN, TRUE,
							  "alwaysOnTop", G_TYPE_BOOLEAN, FALSE,
							  "dockWindow", G_TYPE_BOOLEAN, FALSE,
							  "dockMode", G_TYPE_ULONG, ( gulong ) 0,
							  "inTaskbarWhenMinimized", G_TYPE_BOOLEAN, TRUE,
							  "inTaskbarWhenMaximized", G_TYPE_BOOLEAN, TRUE,
							  "inPager", G_TYPE_BOOLEAN, TRUE,
							  "showStarInTitle", G_TYPE_BOOLEAN, TRUE,
							  "showOwnersInTitle", G_TYPE_BOOLEAN, TRUE,
							  "showWindowOnAll", G_TYPE_BOOLEAN, FALSE,
							  "useDoubleClick", G_TYPE_BOOLEAN, FALSE,
							  NULL);

	
	// defaults for the startup
	config->installProperties("startup",
							  "autoLogonEnabled", G_TYPE_BOOLEAN, FALSE,
							  "autoLogonStatus", G_TYPE_ULONG, ( gulong ) 0,
							  "autoLogonInvisible", G_TYPE_BOOLEAN, FALSE,
							  NULL);
	
	
	GDK_COLOR_SET(&conv_messageCol, 0x0000, 0x0000, 0xFFFF);
	GDK_COLOR_SET(&conv_typingCol, 0xF300, 0xF500, 0x2500);
	
	
	// defaults for the appearance
	config->installProperties("appearance",
							  "containerMode", G_TYPE_ULONG, ( gulong ) CM_MESSAGEWINDOW,
							  "showTrayIcon", G_TYPE_BOOLEAN, TRUE,
							  //"reverseHistory", G_TYPE_BOOLEAN, FALSE,
							  // defaults for the typing notification
							  "tabMessageColor", ICQND_TYPE_COLOR, &conv_messageCol, 
							  "tabTypingColor", ICQND_TYPE_COLOR, &conv_typingCol,	
							  // defaults for the themes
					  		   "EventTheme", G_TYPE_STRING, "default",
					  		   "SmileyTheme", G_TYPE_STRING, "default",
					  		   "StatusTheme", G_TYPE_STRING, "default",
					  		   "ExtendedTheme", G_TYPE_STRING, "default",
					  		   NULL);
							   
	config->installProperties("autoresponses",
							  "autoAway", G_TYPE_ULONG, ( gulong ) 0,
							  "autoNA", G_TYPE_ULONG, ( gulong ) 0,
							  "autoOffline", G_TYPE_ULONG, ( gulong ) 0,
							  "autoAwayEnabled", G_TYPE_BOOLEAN, FALSE,
							  "autoNAEnabled", G_TYPE_BOOLEAN, FALSE,
							  "autoOfflineEnabled", G_TYPE_BOOLEAN, FALSE,
							  "autoAwayMessage", G_TYPE_ULONG, ( gulong ) 0,
							  "autoNAMessage", G_TYPE_ULONG, ( gulong ) 0,
							  NULL);
	
	
	// defaults for conversations
	config->installProperties("conversations",
							  "standardEncoding", G_TYPE_STRING, "UTF-8",
							  "sendTyping", G_TYPE_BOOLEAN, TRUE,
							  "showSmileys", G_TYPE_BOOLEAN, TRUE,
							  "showURIs", G_TYPE_BOOLEAN, TRUE,
							  "showHistory", G_TYPE_BOOLEAN, TRUE,
							  "historyCount", G_TYPE_ULONG, ( gulong ) 5,
							  "colorsSet", G_TYPE_BOOLEAN, FALSE,
							  "foregroundColor", ICQND_TYPE_COLOR, NULL,
							  "backgroundColor", ICQND_TYPE_COLOR, NULL,
							  // defaults for the toolbar
							  "remoteToolbarRules", G_TYPE_STRING, "1,2,20,3,18,4,5,6",
							  "localToolbarRules", G_TYPE_STRING, "7,8,9,10,18,11,12",
							  "buttonbarRules", G_TYPE_STRING, "17,13,14,20,16,15",
							  "remoteToolbarSpacing", G_TYPE_ULONG, ( gulong ) 2,
							  "localToolbarSpacing", G_TYPE_ULONG, ( gulong ) 4,
							  "buttonbarSpacing", G_TYPE_ULONG, ( gulong ) 0,
							  "showRemoteToolbar", G_TYPE_BOOLEAN, TRUE,
							  "showLocalToolbar", G_TYPE_BOOLEAN, TRUE,
							  "showButtonbar", G_TYPE_BOOLEAN, TRUE,	
							  "messageTag", G_TYPE_STRING, "%t %a: |||",
							  "dontApplyColors", G_TYPE_BOOLEAN, FALSE,	
							  "alwaysViaServer", G_TYPE_BOOLEAN, FALSE,	
							  "sendByReturn", G_TYPE_BOOLEAN, FALSE, 							  
							  NULL); 
							  							  
	
	// defaults for file transfer
	config->installProperties("filetransfer",
							  "lastDirectory", 	G_TYPE_STRING, getenv("HOME"),
							  NULL);
							  
	// defaults for the contact list
	GDK_COLOR_SET(&cl_selCol, 0xF000, 0xF000, 0xF000);
	GDK_COLOR_SET(&cl_vlCol, 0x6800, 0x4C00, 0x0000);
	GDK_COLOR_SET(&cl_noCol, 0x9100, 0x6900, 0x5500);
	GDK_COLOR_SET(&cl_idleCol, 0x7777, 0x7777, 0x7777);
	GDK_COLOR_SET(&cl_wentOnlineCol, 0x9999, 0x0000, 0x0000);
	GDK_COLOR_SET(&cl_groupCol, 0xEEEE, 0xEEEE, 0xEEEE);

	config->installProperties("contactlist",
							  "showOfflineUsers", G_TYPE_BOOLEAN, TRUE,
							  "showGroups", G_TYPE_BOOLEAN, TRUE,
							  "showEmptyGroups", G_TYPE_BOOLEAN, TRUE,
							  "lastOpenedGroups", G_TYPE_ULONG, ( gulong ) 0, 
							  "blinkEvents", G_TYPE_BOOLEAN, TRUE,
							  "highlightOnline", G_TYPE_BOOLEAN, TRUE,
							  "highlightARChecked", G_TYPE_BOOLEAN, TRUE,
							  "showTooltips", G_TYPE_BOOLEAN, TRUE,
							  "tooltipsMask", G_TYPE_ULONG, ( gulong ) 0xFFFFFFFF,
							  "showRealNames", G_TYPE_BOOLEAN, FALSE,
							  "showExtendedIcons", G_TYPE_BOOLEAN, TRUE,
							  "extendedIconsMask", G_TYPE_ULONG, ( gulong ) 63, 
							  "showColors", G_TYPE_BOOLEAN, TRUE,
							  "selectionColor", ICQND_TYPE_COLOR, &cl_selCol,
							  "contactVListColor", ICQND_TYPE_COLOR, &cl_vlCol,
							  "contactNoMsgColor", ICQND_TYPE_COLOR, &cl_noCol,
							  "contactIDLEColor", ICQND_TYPE_COLOR, &cl_idleCol,
							  "contactONColor", ICQND_TYPE_COLOR, &cl_wentOnlineCol,
							  "contactGroupsColor", ICQND_TYPE_COLOR, &cl_groupCol,
							  "sortRulesOrder", G_TYPE_STRING, "0, 1, 2, 3, 4",
							  "sortRulesEnabled", G_TYPE_STRING, "1, 1, 1, 1, 1",
							  "contactsFont", G_TYPE_STRING, "",
							  "contactsUseOwnFont", G_TYPE_BOOLEAN, FALSE,
							  NULL);
							  
	config->installProperties("applications",
							  "useGnomeDefaults", G_TYPE_BOOLEAN, TRUE,
							  "browserCommand", G_TYPE_STRING, "DUMMY",
							  "browserMode", G_TYPE_ULONG, ( gulong ) WMODE_TAB,
							  NULL);
							  
	// check whether the browserCommand-entry was just initialized
	config->getProperties("applications", "browserCommand", &browser, NULL);
	if (!browser || !*browser || !strcmp(browser, "DUMMY"))
	{
		if (browser)
			g_free(browser);
		config->setProperties(FALSE, "applications", 
							  "browserCommand", (browserInfo = u_getFirstInstalledBrowser())?browserInfo->command: "",
							  NULL);
	}						  
							  	
	// setup the icons
	i_getIcons();
	
	
	return TRUE;
}




IMOwner::~IMOwner()
{
	settings	*config;
	
	// save and delete the config
	config = settings_getSettings();
	
	delete window;
	
	config->writeSettings();
}



gboolean IMOwner::openEvent(userEventType evType, guint status)
{
	basicWindow	*newEvent;
	IMEventManager *newMan;
	IMBuddy *buddy;
	IMOwnerDaemon	*owner;
	GList *search;
	
	newMan = NULL;
	newEvent = NULL;
	
	// set a new autoresponse
	if (evType == UE_AUTORESPONSESET)
	{
		
		if (!IO_getOwnerList())
			return FALSE;
		
		buddy = (IMBuddy*)((IMUserDaemon*)IO_getOwnerList()->data)->dowry;
	
		newEvent = buddy->getOpenedEvent(evType);	
		
		if (newEvent)
		{
			((autoResponseWindow*)newEvent)->updateStatus(status);
			newEvent->focusWindow();
		} else
		{
			newEvent = (basicWindow*)new autoResponseWindow(status);
			newMan = new IMAutoResponseManager(buddy->user->info, TRUE);
			newMan->addCallback((IMEventCallback)newEvent->cb_eventCallback, newEvent);
			newEvent->setDestroyCallback((IMEventCallback)this->cb_buddyCallback, this);

			newEvent->manager = newMan;
			buddy->user->addManager(newMan);				
			newEvent->createWindow();
			buddy->openEvents = g_list_append(buddy->openEvents, newEvent);
		}	

		return TRUE; 		
	} 
	/*
	if (evType == UE_USERINFO)
	{
		if (!protocols)
			return FALSE;
		
		buddy = (IMBuddy*)protocols->data;
		
		newEvent = buddy->getOpenedEvent(evType);
		
		if (!newEvent)
		{
			newEvent = (basicWindow*)new userInfoWindow();
			newMan = new IMUserUpdateManager(buddy->user->info);
			newMan->setCallback((IMEventCallback)newEvent->cb_eventCallback, newEvent);
			newEvent->setDestroyCallback((IMEventCallback)this->cb_buddyCallback, this);

			newEvent->manager = newMan;
			buddy->user->addManager(newMan);
			newEvent->createWindow();
			buddy->openEvents = g_list_append(buddy->openEvents, newEvent);
		} else
		newEvent->focusWindow();
		
		return TRUE;
	} else */
	
	
	// these are icq specific things	
	if (evType == UE_WHITEPAGES ||
		evType == UE_SETRANDOMCHAT ||
		evType == UE_STARTRANDOMCHAT)
	{
		search = IO_getOwnerList();
		while(search)
		{
			owner = (IMOwnerDaemon*)search->data;
			
			if (owner->info->pluginID == LICQ_PPID)
				break;
			
			search = search->next;
		}			
		
		// We have an icq account
		if (search)
		{
			newEvent = ((IMBuddy*)owner->dowry)->getOpenedEvent(UE_WHITEPAGES);
			if (newEvent)
				newEvent->focusWindow();
			else
			{
				switch (evType)
				{
					case UE_WHITEPAGES:
						newEvent = (basicWindow*)new searchForUserDialog();
						newMan = new IMUserSearchManager(owner->info);
						break;
								
					case UE_SETRANDOMCHAT:
						newEvent = new selectRandomChatGroupWindow();
						newMan = new IMRandomChatManager(owner->info);
						break;
					
					case UE_STARTRANDOMCHAT:
						newEvent = new startRandomChatWindow();
						newMan = new IMRandomChatManager(owner->info);
						break;
					
					default: break;
				}
				
				newMan->addCallback((IMEventCallback)newEvent->cb_eventCallback, newEvent);
				newEvent->setDestroyCallback((IMEventCallback)this->cb_buddyCallback, this);

				newEvent->manager = newMan;
				owner->addManager(newMan);
				newEvent->createWindow();
				((IMBuddy*)owner->dowry)->openEvents = g_list_append(((IMBuddy*)owner->dowry)->openEvents, newEvent);
			}
		} else
		u_showAlertMessage("Not available", "This feature is only available for the ICQ protocol.", GTK_STOCK_DIALOG_ERROR);

		return TRUE;		
	}

	
	// other general events
	newEvent = getOpenedEvent(evType);
	
	if (newEvent)
	{
		newEvent->focusWindow();
		return TRUE;
	}
		
	switch(evType)
	{			
		case UE_AUTHORIZATIONGRANT:
			newEvent = (basicWindow*)new IDActionWindow(UE_AUTHORIZATIONGRANT, 
														(IDActionCallback)this->cb_idActionCallback, 
														this, 
														tr("Please enter the user id (and the protocol) for the user that you want to send an authorization."), 
														tr("Send authorization"), 
														TRUE, 
														NULL, 
														FALSE,
														tr("Send Authorization"),
														"auth.png");
			break;
		
		case UE_AUTHORIZATIONREQUEST:
			newEvent = (basicWindow*)new IDActionWindow(UE_AUTHORIZATIONREQUEST, 
														(IDActionCallback)this->cb_idActionCallback, 
														this, 
														tr("Please enter the user id (and the protocol) for the user that you want to send an authorization request."), 
														tr("Send authorization request"), 
														TRUE, 
														NULL, 
														FALSE, 
														tr("Send Request"),
														"auth.png");
			break;
		
		case UE_ADDUSER:
			newEvent = (basicWindow*)new IDActionWindow(UE_ADDUSER, 
														(IDActionCallback)this->cb_idActionCallback, 
														this, 
														tr("Please enter the user id (and the protocol) for the user that you want to add to your contact list."), 
														tr("Add a user"), 
														FALSE, 
														tr("_Alert user"), 
														TRUE, 
														tr("Add user"), 
														"adduser.png");
			break;
		
		case UE_EDITGROUPS:
			newEvent = (basicWindow*)new groupsWindow((groupsWindowCallback)this->cb_groupWindowCallback, this);
			break;
		
		case UE_REGISTEROWNER:
			newEvent = new newOwnerWindow("Add a New User Account to Licq");
			break;
		
		case UE_MANAGEOWNERS:
			newEvent = new ownerManagerWindow();
			break;
		
		case UE_LOGMESSAGES:
			newEvent = new logWindow(logDaemon);
			break;
		
		case UE_SETOWNERPROPERTIES:
			u_showAlertMessage("Sorry", "Not implemented yet", GTK_STOCK_DIALOG_WARNING);
			break;
		
		case UE_OPTIONS:
			newEvent = new optionsWindow();
			break;
		
		case UE_USERLISTS:
			newEvent = new listsWindow(this);
			break;
		
		case UE_STATISTICS:
			newEvent = new statisticsWindow();
			break;
		
#ifdef HAVE_LIBGPGME
		case UE_GPGMANAGER:
			newEvent = new GPGManagerWindow();
			break;
#endif
		
		default:
			// IMBuddy::openEvent(evType, FALSE); -> THIS WILL CRASH
			break;
	}
	
	if (newEvent)
	{
		newEvent->setDestroyCallback((IMEventCallback)this->cb_buddyCallback, this);
		newEvent->createWindow();
		openEvents = g_list_append(openEvents, newEvent);
	}
	
		
	return TRUE;
}



IMBuddy* IMOwner::getBuddyForLastGlobalEvent(gboolean favourSystemEvents, basicEventInfo** info)
{
	IMBuddy			*focus;
	IMUserDaemon	*tmpFocus, 
					*findFocus;
	GList			*search;
	basicEventInfo	*tmpEvent, 
					*basicEvent;
	time_t	 		t;

	
	t = 0;
	findFocus = NULL;
	basicEvent = NULL;
	search = IO_getOwnerList();
	
	while(search)
	{
		if ((tmpFocus = ((IMOwnerDaemon*)search->data)->getLastEvent(&tmpEvent, TRUE)) && tmpEvent->time > t)
		{					
			findFocus = tmpFocus;
			basicEvent = tmpEvent;
		}
				
		search = search->next;
	}
			
	// a recent event was found
	if (findFocus)
	{
		focus = (IMBuddy*)findFocus->dowry;
				
		// a user and no buddy around it? this must be a new user -> create a buddy
		if (!focus && !findFocus->info->onIgnoreList)
		{
			focus = new IMBuddy(findFocus);
			findFocus->dowry = focus;
		}	
		
		if (info)
			*info = basicEvent;
		
		return focus;
	}
	
	return NULL;
}



void IMOwner::setStatus(guint status, gboolean promptAR)
{
	GList			*owners;
	gboolean		showConnect;
	IMOwnerDaemon	*owner;
	
	
	owners = IO_getOwnerList();
	
	showConnect = FALSE;
	while(owners)
	{
		owner = (IMOwnerDaemon*)owners->data;
		
		// do we want to go invisible?
		if (status == BUDDY_STATUS_INV)
		{
			if (!owner->info->isInvisible)
				owner->setStatus(BUDDY_STATUS_ONLINE);			
			owner->setInvisible(!owner->info->isInvisible);
		} else
			owner->setStatus(status);
		
		// if we go offline unset invisibility
		if (status == BUDDY_STATUS_OFFLINE && owner->info->isInvisible)
			owner->setInvisible(FALSE);
		
		// eventually start the connection animation on the proto button
		if (owner->info->status == BUDDY_STATUS_OFFLINE && status != BUDDY_STATUS_OFFLINE)
			window->updateStatusButton(owner, BUDDY_STATUS_CONNECT, FALSE);
		else
			window->updateStatusButton(owner, owner->info->status, owner->info->isInvisible);
		
		if (owner->info->status == BUDDY_STATUS_OFFLINE && status != BUDDY_STATUS_OFFLINE)
			showConnect = TRUE;
		
		owners = owners->next;
	}
	
	// check whether the auto response window should be prompted
	if (promptAR && 
		status != BUDDY_STATUS_ONLINE && 
		status != BUDDY_STATUS_OFFLINE &&
		status != BUDDY_STATUS_INV)
		openEvent(UE_AUTORESPONSESET, status);
	

	if (showConnect && status != BUDDY_STATUS_OFFLINE)
		window->updateStatus(BUDDY_STATUS_CONNECT, FALSE);
	else
	if (showConnect && status == BUDDY_STATUS_OFFLINE)
		window->updateStatus(BUDDY_STATUS_OFFLINE, FALSE);		
}



/*

void IMOwner::setStatus(guint status)
{
	GList 	*search;
	gboolean someoneOffline;
	
	// ask all protocols to change the status
	search = protocols;
	
	someoneOffline = FALSE;
	
	while(search)
	{
		if ( ((IMBuddy*)search->data)->user->info->status == BUDDY_STATUS_OFFLINE)
			someoneOffline = TRUE;
	
		((IMOwnerDaemon*)((IMBuddy*)search->data)->user)->setStatus(status);
					
		search = search->next;
	}
	
	if (someoneOffline && status != BUDDY_STATUS_OFFLINE)
		window->updateStatus(BUDDY_STATUS_CONNECT, FALSE);
	else
	if (someoneOffline && status == BUDDY_STATUS_OFFLINE)
		window->updateStatus(BUDDY_STATUS_OFFLINE, FALSE);		
}


void IMOwner::setInvisible(gboolean setting)
{
	GList 		*search;
	gboolean 	goOnline;
	// ask all protocols to change the status
	search = protocols;
	
	goOnline = FALSE;
	
	while(search)
	{
		if ( ((IMBuddy*)search->data)->user->info->status == BUDDY_STATUS_OFFLINE)
		{
			goOnline = TRUE;
			((IMOwnerDaemon*)((IMBuddy*)search->data)->user)->setStatus(BUDDY_STATUS_ONLINE);
		}

		((IMOwnerDaemon*)((IMBuddy*)search->data)->user)->setInvisible(setting);		
					
		search = search->next;
	}
	
	if (goOnline)
		window->updateStatus(BUDDY_STATUS_CONNECT, FALSE);			
} */



gboolean IMOwner::autoConnect()
{
	gulong		status;
	gboolean	enabled,
				goInvisible;
	GList		*search;
	
	settings_getSettings()->getProperties("startup",
										  "autoLogonEnabled", &enabled,
							  			  "autoLogonStatus", &status,
							  			  "autoLogonInvisible", &goInvisible,
										  NULL);
	
	if (!enabled)
		return TRUE;
	
	if (goInvisible)
	{
		for (search = IO_getOwnerList(); search; search = search->next)
			((IMOwnerDaemon*)search->data)->setInvisible(TRUE);
	}	
	
	setStatus(status, FALSE);
	
	return TRUE;	
}

gint blub_;

gboolean testbla(IMBuddy* buddy)
{	
	/*
	switch(rand() % 6)
	{
		case 0:
			buddy->user->info->status = BUDDY_STATUS_OFFLINE;
			break;
		
		case 1:
			buddy->user->info->status = BUDDY_STATUS_AWAY;
			break;
		
		case 2: 
			buddy->user->info->typing = 2;
			break;
		
		case 3:
			buddy->user->info->typing = 0;
			break;
		
		case 4:
			buddy->user->info->status = BUDDY_STATUS_NA;
			break;
		
		case 5: 
			buddy->user->info->status = BUDDY_STATUS_ONLINE;
			break;
	}
	
	
	buddy->updateContactListStatus(rand()%2);
	buddy->updateContactListStates();
	*/
	return TRUE;
}


void IMOwner::rebuildContactList(gboolean dispatchEvents)
{
	GList				*search,
						*searchOwners,
						*searchGroups;
	IMOwnerDaemon		*owner;
	IMUserDaemon		*user;
	IMBuddy				*buddy;
	groupInfo			*group;
	contactListEntry	*groupEntry;
	contactListGroup	*tmpGroup;
	basicEventInfo		*basicEvent;
	gboolean			useRealNames;
	
	// shall we use real names?
	settings_getSettings()->getProperties("contactlist",
										  "showRealNames", &useRealNames,
										  NULL);
										  
		
	// clear the old contact list
	if (window->contacts->getChildren())
	{
		// save the old opening state
		searchGroups = window->contacts->getAllChildrenOfType(CE_GROUP);
		search = searchGroups;
		while(search)
		{
			tmpGroup = (contactListGroup*)search->data;
			group = (groupInfo*)tmpGroup->data;
			// does this group still exist?
			if (IO_getGroupManager()->getGroupByIcqndID(tmpGroup->id))
				group->opened = tmpGroup->isGroupOpen();
			search = search->next;
		}
		g_list_free(searchGroups);
	}
	
	window->contacts->deleteAllChildren();
	
	if (!IO_getOwnerList())
		return;
	
	
	// build the groups
	search = IO_getGroupManager()->groups;
	while(search)
	{
		group = (groupInfo*)search->data;
		groupEntry = new contactListGroup(group->name, group->icqndID, group);
		((contactListGroup*)groupEntry)->openGroup(group->opened);
		window->contacts->addChild(groupEntry);
		search = search->next;
	}
		
	// rebuild the contact list
	searchOwners = IO_getOwnerList();
	while(searchOwners)
	{
		owner = (IMOwnerDaemon*)searchOwners->data;	
		
		fprintf(stderr, "IMOwner::rebuildContactList(): Rebuild list for %s\n", owner->info->alias);
		
		search = owner->users;
		while(search)
		{
			user = (IMUserDaemon*)search->data;		
			
			if (user->info->onIgnoreList)
			{
				search = search->next;
				continue;
			}
			
			if (!user->dowry)
				buddy = new IMBuddy(user);
			else
				buddy = (IMBuddy*)user->dowry;
			
			g_list_free(buddy->cEntries);
			buddy->cEntries = NULL;
			
			buddy->putUserIntoGroups(window->contacts, useRealNames);
			
			// update events if there are some
			if (dispatchEvents)
				user->dispatchPendingEvents();
			
			basicEvent = user->getEvent(EO_LAST | EO_SYSTEM | EO_NORMAL);

			if (basicEvent)
				buddy->updateContactListEvents(basicEvent, TRUE);
			
			/*if (!strcmp(user->info->licqID, "154400962"))
				g_timeout_add(100, (GSourceFunc)testbla, buddy); */
			
			search = search->next;
		}
		
		searchOwners = searchOwners->next;
	}	
	
	window->contacts->realizeEntry(TRUE);
}



void IMOwner::updateAllUsers()
{
	gint 			i;
	GList			*owners,
					*users,
					*entries;
	IMBuddy			*buddy;
	IMUserDaemon	*user;
	
	// check whether an old update process wasn't finished and finish it
	for (owners = IO_getOwnerList(); owners; owners = owners->next)
	{
		for (users = ((IMOwnerDaemon*)owners->data)->users; users; users = users->next)
		{
			user = (IMUserDaemon*)users->data;
			user->removeManagerByCallback((gpointer)this->cb_massEventCallback);			
			buddy = (IMBuddy*)user->dowry;
			for (entries = buddy->cEntries; entries; entries = entries->next)
				((contactListUser*)entries->data)->setBackgroundColorEnabled(FALSE);
			
		}
	
	}
	
	// start a new update
	updateTime = time(NULL);
	
	for (i=0;i<2;i++)
		updateOneUser(window->contacts);
}



gboolean IMOwner::updateOneUser(contactListEntry* pos)
{
	IMBuddy				*buddy;
	contactListUser		*entry;
	GList				*usearch,
						*esearch;
	IMUserUpdateManager *uman;
	GdkColor			col;
	
	usearch = pos->getChildren();
	
	settings_getSettings()->getProperties("contactlist", 
										  "selectionColor", &col,
										  NULL);
	
	while(usearch)
	{
		entry = (contactListUser*)usearch->data;
		
		if (entry->type == CE_USER)
		{
			buddy = (IMBuddy*)((IMUserDaemon*)entry->data)->dowry;
			if (buddy->user->info->lastUpdated < updateTime)
			{
				esearch = buddy->cEntries;
				while(esearch)
				{
					entry = (contactListUser*)esearch->data;
					entry->setBackgroundColor(col);
					entry->setBackgroundColorEnabled(TRUE);
					esearch = esearch->next;
				}
				
				buddy->user->info->lastUpdated = updateTime;
				uman = new IMUserUpdateManager(buddy->user->info);
				uman->addCallback((IMEventCallback)this->cb_massEventCallback, this);
				buddy->user->addManager(uman);
				uman->updateUserInfo(buddy->user->info->keepAlias? buddy->user->info->alias: NULL);
				return TRUE;
			}
		} else
		if (updateOneUser(entry))
			return TRUE;

		usearch = usearch->next;
	}
	
	return FALSE;
}





gboolean IMOwner::cb_ownerCallback(userEventType type, 
								   gint command, 
								   gint lastResult, gpointer info, 
								   IMOwner* self)
{
	GList					*search,
							*search2;
	basicEventInfo			*lastEv;
	contactListEntry		*entry;
	IMBuddy					*newBuddy;
	IMAutoResponseManager	*aman;
	IMUserDaemon			*user;
	IMOwnerDaemon			*owner;
	gchar					*txt,
							*txt2;
	gint					newStatus;
	gboolean				newInvisible,
							useRealName;

	
	// evaluate the command
	switch (command)
	{
		case EC_FROMMANAGER:
			if (info)
				((IMBuddy*)((IMUserDaemon*)info)->dowry)->openEvent((userEventType)lastResult, 0);
			else
				self->openEvent((userEventType)lastResult, FALSE);
			break;
		
		case UC_UPDATEMODES:
			user = (IMUserDaemon*)info;
			newBuddy = (IMBuddy*)user->dowry;
		
			if (!newBuddy)
				break;
		
			search = newBuddy->cEntries;
			while(search)
			{
				((contactListUser*)search->data)->unrealizeEntry();
				delete (contactListUser*)search->data;
				search = search->next;
			}
			g_list_free(newBuddy->cEntries);
			newBuddy->cEntries = NULL;
			settings_getSettings()->getProperties("contactlist", 
												  "showRealNames", &useRealName,
										  		  NULL);
			
			// if "lastResult" is set the user is invisible now
			newBuddy->putUserIntoGroups(self->window->contacts, useRealName);
			
			// realize elements
			for (search = newBuddy->cEntries; search; search = search->next)
				((contactListEntry*)search->data)->realizeEntry(TRUE);
		
			// overtake old events
			lastEv = newBuddy->user->getEvent(EO_LAST | EO_SYSTEM | EO_NORMAL);
			
			// Update new group data
			search = self->window->contacts->getAllChildrenOfType(CE_GROUP);
			for (search2 = search; search2; search2 = search2->next)
				((contactListGroup*)search2->data)->updateEntry(FALSE);

			g_list_free(search);

			if (lastEv)
				newBuddy->updateContactListEvents(lastEv, TRUE);
			break;
			
		
		case OC_NEWUSER:
		case OC_LOADUSER:	
		case OC_NEWUSER_NOGROUP:
			user = (IMUserDaemon*)info;
		
			if (user->info->onIgnoreList)
				break;
		
			// this is somehow complicated: normally a buddy is wrapped around
			// a user to take over callback stuff, open events ...
			// however, when there was a system message ("a user added you")
			// the buddy is already created by OC_UNKNOWNUSER, so don't create
			// it again!
			if (!user->dowry)
				newBuddy = new IMBuddy(user);
			else
				newBuddy = (IMBuddy*)user->dowry;
			
			settings_getSettings()->getProperties("contactlist", 
												  "showRealNames", &useRealName,
										  		  NULL);
			
			if (command != OC_NEWUSER_NOGROUP)
				newBuddy->putUserIntoGroups(self->window->contacts, useRealName);	

			// realize elements
			for (search = newBuddy->cEntries; search; search = search->next)
				((contactListEntry*)search->data)->realizeEntry(TRUE);
			
			// check for old events that were not deleted last time
			newBuddy->user->dispatchPendingEvents();
			
			// self->window->updateStatus(newBuddy->user->info->status);			
			break;
			
/*			
		case OC_NEWGROUP:
			if (!(newGroup = (contactListGroup*)self->window->contacts->findEntryByID((gchar*)info, CE_GROUP)))
			{		
				newGroup = new contactListGroup((gchar*)info, 
											    NULL);
				if (lastResult != 0)
					newGroup->setDefault(TRUE);
		
				self->window->contacts->addChild(newGroup);
			}
			break;
	*/	
		
		case OC_REMOVEUSER:
			user = (IMUserDaemon*)info;
			if (!user)
				break;
			
			newBuddy = (IMBuddy*)user->dowry;
			if (!newBuddy)
				break;
			
			search = newBuddy->cEntries;
			
			// delete that user from the contact list
			while(search)
			{
				entry = (contactListEntry*)search->data;
				delete entry;
				
				search = search->next;
			}	
			
			// decrease the group count
			search = IO_getGroupManager()->groups;
			while(search)
			{
				search2 = user->groups;
				
				while(search2)
				{
					if (search2->data == ((groupInfo*)search->data)->icqndID)
						((groupInfo*)search->data)->numUsers--;						
					
					search2 = search2->next;
				}
				
				search = search->next;
			}
			
			newBuddy->closeAllEvents();
			
			fprintf(stderr, "IMOwner::cb_ownerCallback(): Removed user %s from contact list\n", user->info->alias);							
			
			delete newBuddy;
			break;
		
		
		case OC_UPDATEUSER:
			break;
		
		
		case OC_SETSTATE:
		case UC_UPDATEUSER:
			owner = (IMOwnerDaemon*)info;
			self->window->updateStatusButton(owner, owner->info->status, owner->info->isInvisible);
		
			// try to make a good "average" status
			newStatus = BUDDY_STATUS_OFFLINE;
		
			search = IO_getOwnerList();
			while(search)
			{
				owner = (IMOwnerDaemon*)search->data;
				
				if (owner->info->status != BUDDY_STATUS_OFFLINE)
				{	
					// if no status is defined set the best one
					if (newStatus == BUDDY_STATUS_OFFLINE)
						newStatus = owner->info->status;
					else
					// more protos are online with different statuses. Take "online" and quit
					if (newStatus != owner->info->status)
					{
						newStatus = BUDDY_STATUS_ONLINE;
						break;
					}
				}
				
				search = search->next;
			}	

			// try to make a good "average" invisible state
			search = IO_getOwnerList();
			newInvisible = FALSE;
			
			while(search)			
			{
				owner = (IMOwnerDaemon*)search->data;
				
				if (owner->info->isInvisible)
				{
					newInvisible = TRUE;
					break;
				}
				
				search = search->next;
			}				
		
			self->window->updateStatus(newStatus, newInvisible);
			/*
			user = (IMUserDaemon*)info;
			// licq sends a logoff before logging on -> ignore that
			if (!(user->info->status == BUDDY_STATUS_OFFLINE && self->window->isTrayAnimation()))
				self->window->updateStatus(user->info->status, user->info->isInvisible);
			*/
			
			if (owner->info->status == BUDDY_STATUS_OFFLINE)
				owner->oldStatus = BUDDY_STATUS_OFFLINE;
			break;	
		
		case OC_LOGON_RATE_ERROR:
			self->window->updateStatus(BUDDY_STATUS_OFFLINE, FALSE);
			u_showAlertMessage(tr("Rate Error"), tr("You have logged on too often in a short time, so the server has blocked you for some mimutes. Please try again later."), GTK_STOCK_DIALOG_WARNING);
			break;
		
		case OC_EVENT_DELETED:
		case OC_EVENT_ADDED:
			

			if (command == OC_EVENT_ADDED)
			{
				// only blink an event if there is no more recent system event
				if (self->ownerEventCount == 0 || ((basicEventInfo*)info)->isOwnerEvent)
					self->window->setTrayBlinkEvent((basicEventInfo*)info);
				
				if (((basicEventInfo*)info)->isOwnerEvent)
					self->ownerEventCount++;
				else
					self->eventCount++;
			} else
			
			{
				// blink the last recent event
				lastEv = NULL;
				self->getBuddyForLastGlobalEvent(TRUE, &lastEv);
				self->window->setTrayBlinkEvent(lastEv);
				
				if ( ((basicEventInfo*)info)->isOwnerEvent)
					self->ownerEventCount--;
				else
					self->eventCount--;
			} 
						
			if (self->ownerEventCount == 0 && self->eventCount == 0)
				self->window->setPendingEventNotify(FALSE);
			else
				self->window->setPendingEventNotify(TRUE);
			
			if (self->ownerEventCount>0)
			{
				txt = g_strdup_printf("<b>%s</b>", tr("System messages"));
				self->window->updateSystemInfo(self->ownerEventCount, txt, "");
				g_free(txt);
			}
			else
				self->window->updateSystemInfo(self->eventCount, tr("Message"), "");
			
			break;
		
			
		// the auto response was set for the first protocol -> set it for all protocols
		// (if we wanted to set this for only one proto IMOwner::cb_ownerCallback wouldn't
		// be called, IMBuddy::cb_buddyCallback)
		case UC_SETAUTORESPONSE:
			if (g_list_length(IO_getOwnerList())>1)
			{
				txt = NULL;
				search = IO_getOwnerList();
				while(search)
				{
					user = (IMUserDaemon*)search->data;
				
					aman = new IMAutoResponseManager(user->info, TRUE);
					user->addManager(aman);
				
					if (!txt)
						txt = aman->getCustomResponse();
					else
						aman->setCustomResponse(txt, FALSE);
				
					delete aman;
				
					search = search->next;
				}
				
				g_free(txt);
			}
			break;
						
		
		default:
			fprintf(stderr, "IMOwner::cb_ownerCallback: unknown new event (No: %d)!\n", command);
			break;
	}
		
	
	return TRUE;
}



void IMOwner::cb_menuCallback(GtkWidget* item, gint menu, gint selection, IMOwner* self)
{
	IMBuddy				*focus;
	IMOwnerDaemon		*ownerDaemon;
	contactListEntry	*focusedEntry;
	basicEventInfo		*basicEvent;
	gint	 			last;
	gboolean			cViewState;
	gchar				*txt;
	GList				*ownersSearch,
						*usersSearch;

	switch(menu)
	{
		// a double click on the notify box
		case MENU_NOTIFYBOX:
			
			// find the most recent event and open it
			if ((focus = self->getBuddyForLastGlobalEvent(TRUE, &basicEvent)))
				focus->openEvent(self->getWindowIDForEvent(basicEvent->type), FALSE); 	
			
			break;
			
			
		// a double click on the user name
		case MENU_DOUBLECLICK:
			// FIXME: search is already done in cb_contactListActivated (mainWindow)
			focusedEntry = (contactListEntry*)self->window->contacts->getFocusedEntry();
			focus = (IMBuddy*)((IMUserDaemon*)focusedEntry->data)->dowry;

			basicEvent = focus->user->getEvent(EO_LAST | EO_SYSTEM | EO_NORMAL);
			if (basicEvent)
				last = basicEvent->type;
			else
				last = -1;
		
			// open the appropriate event for the last event (or open message window)
			focus->openEvent(self->getWindowIDForEvent(last), TRUE);
			break;
		
			
		// the status menu		
		case MENU_STATUS:
			// the owner was stored in the parent menu by mainWindow::addStatusButton
			if (item && gtk_widget_get_parent(item))
				ownerDaemon = (IMOwnerDaemon*)g_object_get_data(G_OBJECT(gtk_widget_get_parent(item)), "icqnd-status-owner");
			else
				ownerDaemon = NULL;
			
			// if an ownerDaemon is set the status change should only affect one protocol
			if (ownerDaemon)
			{
				if (selection != BUDDY_STATUS_INV)
					ownerDaemon->setStatus(selection);
				else
					ownerDaemon->setInvisible(!ownerDaemon->info->isInvisible);
				
				if (ownerDaemon->info->status == BUDDY_STATUS_OFFLINE && selection != BUDDY_STATUS_OFFLINE)
					self->window->updateStatusButton(ownerDaemon, BUDDY_STATUS_CONNECT, ownerDaemon->info->isInvisible);
				
				if (selection != BUDDY_STATUS_ONLINE && 
					selection != BUDDY_STATUS_OFFLINE &&
					selection != BUDDY_STATUS_INV) 
				((IMBuddy*)ownerDaemon->dowry)->openEvent(UE_AUTORESPONSESET, FALSE);
			} else
				self->setStatus(selection, TRUE);
			break;
			
			
		// the user menu
		case MENU_USER:
			// get the selected entry
			focusedEntry = (contactListEntry*)self->window->contacts->getFocusedEntry();
			if (!focusedEntry)
				return;
			
			focus = (IMBuddy*)((IMUserDaemon*)focusedEntry->data)->dowry;
			
			if (!focus || !focus->user)
				return;
			
			switch(selection)
			{
				// open the last event
				case MENU_OPENEVENT:
				basicEvent = focus->user->getEvent(EO_LAST | EO_SYSTEM | EO_NORMAL);
				if (basicEvent)
					focus->openEvent(self->getWindowIDForEvent(basicEvent->type), TRUE);
				break;
				
				// remove a user
				case MENU_REMOVEUSER:
					txt = g_strdup_printf(tr("Do you really want to remove %s from your contact list?\n\nThis will delete the history for this user as well"), focus->user->info->alias);
					if (u_showYesNoDialog(tr("Delete a user?"), txt, GTK_STOCK_DIALOG_WARNING, FALSE) == GTK_RESPONSE_YES)
						((IMOwnerDaemon*)focus->user->owner)->removeUserFromList(focus->user);
					break;
				
				// features that have their own windows
				default:
					if (uu_hasCapat(((IMOwnerDaemon*)(focus->user->owner))->protocol->functions, (userEventType)selection))
						focus->openEvent((userEventType)selection, FALSE);
					break;
			}
			break;
			
			
		// the main menu elements ("System", "User")
		case MENU_SYSTEM:
			switch(selection)
			{
				// show offline users
				case MENU_SHOWOFFLINEUSERS:
					cViewState = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(item));	
					settings_getSettings()->setProperties(TRUE, "contactlist",
														  "showOfflineUsers", cViewState,
														  NULL);
					break;
					
				// show groups
				case MENU_SHOWGROUPS:
					cViewState = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(item));	
					settings_getSettings()->setProperties(TRUE, "contactlist",
														  "showGroups", cViewState,
														  NULL);		
					break;

				// show empty groups
				case MENU_SHOWEMPTYGROUPS:
					cViewState = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(item));	
					settings_getSettings()->setProperties(TRUE, "contactlist",
														  "showEmptyGroups", cViewState,
														  NULL);	
					break;
				
				// open all events
				case MENU_OPENALL:
					for (ownersSearch = IO_getOwnerList(); ownersSearch; ownersSearch = ownersSearch->next)
						for (usersSearch = ((IMOwnerDaemon*)ownersSearch->data)->users; usersSearch; usersSearch = usersSearch->next)
						{
							basicEvent = ((IMUserDaemon*)usersSearch->data)->getEvent(EO_LAST | EO_SYSTEM | EO_NORMAL);
							if (basicEvent)
							{
								focus = (IMBuddy*)((IMUserDaemon*)usersSearch->data)->dowry;
								focus->openEvent(self->getWindowIDForEvent(basicEvent->type), TRUE);
							}
						}			
					break;
						
				case MENU_ABOUT:
					self->window->showAboutWindow();
					break;
				
				case MENU_QUIT:
					self->window->destroyWindow();
					break;	
				
				// update all users
				case MENU_UPDATEALL:
					self->updateAllUsers();
					break;		
					
				// features that have their own window
				default:
					self->openEvent((userEventType)selection, 0);
					break;
				
			}
			break;
	}		
}


void IMOwner::cb_droppingCallback(contactListEntry *entry, guint type, GtkSelectionData *dragData, IMOwner* self)
{
	basicWindow			*wnd;
	IMBuddy				*buddy;
	IMOwnerDaemon		*owner;
	IMUserDaemon		*dragUser;
	gchar				*id,
						*alias;
	gint				res;
	groupInfo			*newGroupInfo,
						*srcGroupInfo;
	contactListGroup	*srcGroup;
	gulong				currentGroups;
	
	// user functions which only work if something is dropped on a user
	if (entry && entry->type == CE_USER)
	{
		buddy = (IMBuddy*)((IMUserDaemon*)(entry->data))->dowry;
	
		switch(type)
		{
			// a string was dropped
			case 0:
				buddy->openEvent(UE_MESSAGE, FALSE);
				wnd = buddy->getOpenedEvent(UE_MESSAGE);
				((conversationWindow*)wnd)->setTypedText((gchar*)dragData->data);
				break;
		
			// a file was dropped
			case 1:
				buddy->openEvent(UE_FILETRANSFERREQUEST, FALSE);
				wnd = buddy->getOpenedEvent(UE_FILETRANSFERREQUEST);
				((fileTransferWindow*)wnd)->addFilesFromURIList((gchar*)dragData->data);
				break;
			
			case 3:
				// FIXME: free id and alias afterwards
				uu_getFromGaimDnDMimeType((gchar*)dragData->data, &id, &alias, &owner, NULL);

				if (!id)
					break;
			
				uu_getUserFromDnDInfo(id, alias, owner, &dragUser, &owner);

				if (!dragUser && !owner)
					break;

				if (!dragUser && owner)
				{
					res = u_showYesNoDialog(tr("User not on list"), tr("The User isn't on your list. Do you want to add this user?"), GTK_STOCK_DIALOG_QUESTION, FALSE);
					
					if (res == GTK_RESPONSE_YES)
					{
						// this is due to a sync problem
						dragUser = owner->addUserTemporally(id);
						owner->addUserToList(id, FALSE);
					}					
				}
				
				if (dragUser)
				{
					buddy->openEvent(UE_CONTACTSREQUEST, FALSE);
					wnd = buddy->getOpenedEvent(UE_CONTACTSREQUEST);
					((contactsWindow*)wnd)->addContact(dragUser);
				}					
				
				break;
			
		}			
	} else
	
	{
		// a contact was dropped on a group
		if (type == 3)
		{
			uu_getFromGaimDnDMimeType((gchar*)dragData->data, &id, &alias, &owner, (gpointer*)&srcGroup);

			if (!id)
				return;
		
			uu_getUserFromDnDInfo(id, alias, owner, &dragUser, &owner);
			
			if (!owner && !dragUser)
				return;
			
			// it's an existing user -> move it to the selected group
			if (dragUser && entry && entry->type == CE_GROUP)
			{		
				//FIXME: better make this with new IMUserDaemon->addGroup and IMUserDaemon->removeGroup
				currentGroups = dragUser->getLicqGroups(GROUPS_USER);
				
				// the user came from another group -> delete him from this group
				if (srcGroup)
				{	
					srcGroupInfo = (groupInfo*)srcGroup->data;
					if (srcGroupInfo->num != 0xFFFF)
						currentGroups = (currentGroups | (1<<srcGroupInfo->num)) % (1<<srcGroupInfo->num);
				}
				
				
				newGroupInfo = (groupInfo*)entry->data;
				
				if (newGroupInfo->num != 0xFFFF)
					currentGroups = currentGroups | (1<<newGroupInfo->num);
				
				dragUser->setLicqGroups(GROUPS_USER, currentGroups);
				dragUser->editUserInformation(TRUE, USER_UPDATE_BASIC);
				
				if (dragUser->owner)
					dragUser->owner->startCallback(UC_UPDATEMODES, 0, dragUser);
				
			}
			
			// the user doesn't exist yet
			// FIXME: move the user into the right group
			if (!dragUser)
			{
				res = u_showYesNoDialog(tr("User not on list"), tr("The User isn't on your list. Do you want to add this user?"), GTK_STOCK_DIALOG_QUESTION, FALSE);
					
				if (res == GTK_RESPONSE_YES)
					owner->addUserToList(id, FALSE);
			}
		}
		
	}
}




void IMOwner::cb_idActionCallback(userEventType type, gint command, gchar* id, guint pluginID, gchar* comment, gboolean toggled, IMOwner* self)
{	
	IMUserDaemon	*userDaemon;
	IMOwnerDaemon	*owner;
	IMEventManager	*man;
	GList			*owners;
	
	
	// remove the window from the list of opened events
	self->openEvents = g_list_remove(self->openEvents, self->getOpenedEvent(type));
	
	if (command != REQUEST_ACCEPTED)
		return;
	
	// find the appropriate owner
	owners = IO_getOwnerList();
	while(owners)
	{
		owner = (IMOwnerDaemon*)owners->data;
		if (owner->info->pluginID == pluginID)
			break;
		owners = owners->next;
	}
	
	if (!owners)
	{
		fprintf(stderr, "IMOwner::cb_idActionCallback: FATAL: couldn't find owner to do a user action!\n");
		return;
	}
	
	
	if (type == UE_ADDUSER)
	{
		owner->addUserToList(id, toggled);
		return;
	}
	
	
	// find our user and create one if neccessary
	userDaemon = owner->findUserByID(id);
	
	if (!userDaemon)
		// okay, the user doesn't exist yet. create a new user
		userDaemon = owner->addUserTemporally(id);

	
	// we have our user now -> execute the appropriate event
	switch(type)
	{
		case UE_AUTHORIZATIONGRANT:
			man = new IMAuthorizationManager(userDaemon->info);
			userDaemon->addManager(man);
			((IMAuthorizationManager*)man)->answerRemoteRequest(TRUE, comment);
			delete man;
			break;
			
		case UE_AUTHORIZATIONREQUEST:
			man = new IMAuthorizationManager(userDaemon->info);
			userDaemon->addManager(man);
			((IMAuthorizationManager*)man)->sendRequest(comment, NULL);
			delete man;
			break;
					
		default:
			fprintf(stderr, "IMOwner::cb_idActionCallback: unknown event to execute on %s (%d)\n", id, type);
			break;
	}
}



gboolean IMOwner::cb_massEventCallback(userEventType type, gint command, gint lastResult, gpointer info, IMOwner* self)
{
	generalEventInfo	*generalEvent;
	contactListEntry	*entry;
	IMBuddy				*buddy;
	IMUserDaemon		*user;
	GList				*search;
	
	
	if (command != UC_EVENT_FINISHED)
		return TRUE;
	
	generalEvent = (generalEventInfo*)info;

	entry = self->window->contacts->findChildByID(generalEvent->basic->userID, CE_USER);
	user = (IMUserDaemon*)entry->data;
	buddy = (IMBuddy*)user->dowry;
	
	if (buddy)
	{
		search = buddy->cEntries;
		while(search)
		{
			((contactListUser*)search->data)->setBackgroundColorEnabled(FALSE);
			search = search->next;
		}
	}
	
	user->removeManagerByCallback((gpointer)self->cb_massEventCallback);
	self->updateOneUser(self->window->contacts);
	
	return TRUE;
}


void IMOwner::cb_ownerSettingsChangedCallback(gchar* sname, GList *values, contactList* self)
{
	settingsProperty	*prop;
	GList				*searchOwners,
						*searchUsers;
	IMUserDaemon		*curUser;
	
	if (!strcmp(sname, "contactlist"))
	{
		for ( ;values; values = values->next)
		{
			prop = (settingsProperty*)values->data;
		
			if (!strcmp(prop->name, "highlightARChecked"))
				ib_highlightARChecked = g_value_get_boolean(prop->val);
			else
			if (!strcmp(prop->name, "highlightOnline"))
				ib_highlightOnline = g_value_get_boolean(prop->val);
		}
	} else
	if (!strcmp(sname, "autoresponse"))
	{
		for ( ;values; values = values->next)
		{
			prop = (settingsProperty*)values->data;
			
			if (!strcmp(prop->name, "autoAway"))
				ib_autoAway = g_value_get_ulong(prop->val);
			else
			if (!strcmp(prop->name, "autoNA"))
				ib_autoNA = g_value_get_ulong(prop->val);
			else
			if (!strcmp(prop->name, "autoOffline"))
				ib_autoOffline = g_value_get_ulong(prop->val);
			else
			if (!strcmp(prop->name, "autoAwayEnabled"))
				ib_autoAwayEnabled = g_value_get_boolean(prop->val);
			else
			if (!strcmp(prop->name, "autoNAEnabled"))
				ib_autoNAEnabled = g_value_get_boolean(prop->val);
			else
			if (!strcmp(prop->name, "autoOfflineEnabled"))
				ib_autoOfflineEnabled = g_value_get_boolean(prop->val);
		}
	} else
	{
		for ( ;values; values = values->next)
		{
			prop = (settingsProperty*)values->data;
			if (!strcmp(prop->name, "alwaysViaServer"))
			{
				// don't automatically send via server
				if (!g_value_get_boolean(prop->val))
				{
					for (searchOwners = IO_getOwnerList(); searchOwners; searchOwners = searchOwners->next)
						for (searchUsers = ((IMOwnerDaemon*)searchOwners->data)->users; searchUsers; searchUsers = searchUsers->next)
						{
							curUser = ((IMUserDaemon*)searchUsers->data);
							
							curUser->info->sendViaServerByUser = FALSE;
							curUser->editUserInformation(TRUE, USER_UPDATE_STATUS, 0);
							curUser->informAllManagers(UC_SENDSERVER, 
													   curUser->info->sendViaServer, 
													   NULL);
						}
				} else
				// always send everything via server
				{
					for (searchOwners = IO_getOwnerList(); searchOwners; searchOwners = searchOwners->next)
						for (searchUsers = ((IMOwnerDaemon*)searchOwners->data)->users; searchUsers; searchUsers = searchUsers->next)
						{
							curUser = ((IMUserDaemon*)searchUsers->data);
							curUser->info->sendViaServerByUser = TRUE;
							curUser->info->sendViaServer = TRUE;
							curUser->informAllManagers(UC_SENDSERVER, TRUE, NULL);
						}				
				}
			}
		}
	}
}




gboolean IMOwner::cb_quitWindowCallback(IMOwner* self)
{
	settings	*config;
	GList		*search,
				*groups;
	gulong		num, 
				i;
	contactListGroup	*contactsGroup;
	
	config = settings_getSettings();
	
	
	// save the opened groups
	num = 0;
	i=0;
	groups = self->window->contacts->getAllChildrenOfType(CE_GROUP);
	search = groups;
	while(search)
	{	
		contactsGroup = (contactListGroup*)search->data;
		if (contactsGroup->isGroupOpen())
			num = num | (1<<i);
		
		i++;
		search = search->next;
	}
	g_list_free(groups);
	
	
	config->setProperties(FALSE, "contactlist", 
						  "lastOpenedGroups", num,
						  NULL);
	
	return FALSE;
}


gboolean IMOwner::cb_checkIDLETimeout(IMOwner* self)
{
#ifdef USE_SCRNSAVER	
	gint 					eventBase,
							errorBase;
	GList					*search,
							*arContents;
	IMOwnerDaemon			*owner;
	gulong					idleTime,
							restoreIDLE,
							autoResponse;
	IMAutoResponseManager	*aman;
	gchar					*tmpAutoResponse;
	
	// try to allocate the idle information
	if (!xIdleInfo)
	{
		if (XScreenSaverQueryExtension(GDK_DISPLAY(), &eventBase, &errorBase))
			xIdleInfo = XScreenSaverAllocInfo();
		else
		{
			fprintf(stderr, "IMOwner::cb_checkIDLETimeout(): could not find idle check extension.\n");
			return FALSE;
		}
	}
	
	// fetch the current idle status
	if (!XScreenSaverQueryInfo(GDK_DISPLAY(), GDK_ROOT_WINDOW(), xIdleInfo))
	{
		fprintf(stderr, "IMOwner::cb_checkIDLETimeout(): could not query idle info\n");
		return FALSE;
	}
	
	// do nothing if no auto status is enabled
	if (!ib_autoOfflineEnabled && !ib_autoAwayEnabled && !ib_autoNAEnabled)
		return TRUE;
		
	// check if we have to change the status
	idleTime = xIdleInfo->idle / 60000;	
	
	// get the highest IDLE status. If a user is active again restore the old
	// status
	restoreIDLE = ib_autoAwayEnabled? ib_autoAway: 0;
	if (ib_autoNAEnabled && ib_autoNA < restoreIDLE)
		restoreIDLE = ib_autoNA;

	for (search = IO_getOwnerList(); search; search = search->next)
	{
		owner = (IMOwnerDaemon*)search->data;
				
		if (owner->oldStatus != BUDDY_STATUS_OFFLINE && idleTime < restoreIDLE)
		{
			owner->setStatus(owner->oldStatus);
			owner->oldStatus = BUDDY_STATUS_OFFLINE;
		}
	}
		
	// check for idle status
	for (search = IO_getOwnerList(); search; search = search->next)
	{
		owner = (IMOwnerDaemon*)search->data;
		
		// check for auto Offline
		if (ib_autoOfflineEnabled &&
			idleTime == ib_autoOffline &&
			owner->info->status != BUDDY_STATUS_OFFLINE)
		{
			owner->oldStatus = BUDDY_STATUS_OFFLINE;
			owner->setStatus(BUDDY_STATUS_OFFLINE);	
		}
		
		// nothing to do if we don't have a status we can go idle from
		if (owner->info->status != BUDDY_STATUS_ONLINE &&
			owner->info->status != BUDDY_STATUS_AWAY &&
			owner->info->status != BUDDY_STATUS_FFC)
			continue;

		
		// check for auto Away
		if (ib_autoAwayEnabled && 
			idleTime == ib_autoAway && 
			(owner->info->status == BUDDY_STATUS_ONLINE || owner->info->status == BUDDY_STATUS_FFC))
		{
			if (owner->oldStatus == BUDDY_STATUS_OFFLINE)
				owner->oldStatus = owner->info->status;
			
			owner->setStatus(BUDDY_STATUS_AWAY);
			
			// set auto response			
			aman = new IMAutoResponseManager(owner->info, TRUE);
			owner->addManager(aman);
			tmpAutoResponse = aman->getCustomResponse();

			arContents = NULL;
			aman->fetchTemplates(BUDDY_STATUS_AWAY, &arContents, NULL);
			
			settings_getSettings()->getProperties("autoresponses",
											  	  "autoAwayMessage", &autoResponse,
											 	  NULL);
			
			if (g_list_length(arContents) > autoResponse)
				aman->setCustomResponse((gchar*)g_list_nth_data(arContents, autoResponse), FALSE);
			
			g_list_foreach(arContents, (GFunc)g_free, NULL);
			g_list_free(arContents);

			delete aman;
			g_free(tmpAutoResponse);		
		}
		
		// check for auto NA
		if (ib_autoNAEnabled &&
			idleTime == ib_autoNA &&
			(owner->info->status == BUDDY_STATUS_ONLINE || owner->info->status == BUDDY_STATUS_FFC || owner->info->status == BUDDY_STATUS_AWAY))
		{
			if (owner->oldStatus == BUDDY_STATUS_OFFLINE)
				owner->oldStatus = owner->info->status;
			
			owner->setStatus(BUDDY_STATUS_NA);
			
			// set auto response			
			aman = new IMAutoResponseManager(owner->info, TRUE);
			owner->addManager(aman);
			tmpAutoResponse = aman->getCustomResponse();

			arContents = NULL;
			aman->fetchTemplates(BUDDY_STATUS_NA, &arContents, NULL);
			
			settings_getSettings()->getProperties("autoresponses",
											  	  "autoNAMessage", &autoResponse,
											 	  NULL);
			
			if (g_list_length(arContents) > autoResponse)
				aman->setCustomResponse((gchar*)g_list_nth_data(arContents, autoResponse), FALSE);
			
			g_list_foreach(arContents, (GFunc)g_free, NULL);
			g_list_free(arContents);

			delete aman;
			g_free(tmpAutoResponse);		
		}	
	}
	
#endif
	return TRUE;
}



void IMOwner::cb_groupWindowCallback(gint command, IMOwner* self)
{
	GList	*searchOwners,
			*search;
	IMUserDaemon* user;
	basicWindow	*lstWnd;
	
	// updating was done -> rebuild EVERYTHING!
	if (command == 1)
	{	
		searchOwners = IO_getOwnerList();
		
		while(searchOwners)
		{
			search = ((IMOwnerDaemon*)searchOwners->data)->users;
			while(search)
			{
				user = (IMUserDaemon*)search->data;
				// FIXME: don't update that much just for the groups
				user->editUserInformation(TRUE, USER_UPDATE_BASIC);
				search = search->next;
			}
			
			searchOwners = searchOwners->next;
		}
		
		self->rebuildContactList(FALSE);
		
		// is the lists window opened? if yes -> rebuild everything
		lstWnd = self->getOpenedEvent(UE_USERLISTS);
		if (lstWnd)
			((listsWindow*)lstWnd)->recreateWindow();
	}	
}



gboolean IMOwner::cb_generalCallback(userEventType type, gint command, gint lastResult, gpointer data, IMOwner* self)
{
	IMOwnerDaemon	*owner;
	IMBuddy			*newProto;
	GList			*ownerSearch,
					*usersSearch;
	gchar			*charset;
	
	switch(command)
	{	
		case EC_FROMMANAGER:
			self->openEvent((userEventType)lastResult, 0);
			break;		
		
		// a new owner was added / removed
		case PC_NEWOWNER:
		case PC_REMOVEOWNER:
			owner = (IMOwnerDaemon*)data;
			if (command == PC_NEWOWNER)
			{
				// wrap a new IMBuddy around that owner
				
				newProto = new IMBuddy((IMUserDaemon*)owner);
				newProto->user->addCallback((IMEventCallback)self->cb_ownerCallback, self);
				newProto->user->dowry = newProto;
				
				settings_getSettings()->getProperties("conversations",
										  			  "standardEncoding", &charset,
										 			  NULL);				
				newProto->user->setCharset(charset, FALSE);

				owner->loadContacts();
		
				owner->dispatchPendingEvents();
		
				// dispatch events of all new users
				usersSearch = owner->users;
				while(usersSearch)
				{
					((IMUserDaemon*)usersSearch->data)->dispatchPendingEvents();
					usersSearch = usersSearch->next;
				}
				
				// if there a now 2 owners active that first one has to be displayed (??)
				if (g_list_length(IO_getOwnerList())==2)
					self->window->addStatusButton((IMOwnerDaemon*)IO_getOwnerList()->data);
				
				if (g_list_length(IO_getOwnerList())>1)
					self->window->addStatusButton((IMOwnerDaemon*)data);
				
			} else
			{					
				// delete all children
				usersSearch = owner->users;
				while(usersSearch)
				{
					if (((IMUserDaemon*)usersSearch->data)->dowry)
						delete (IMBuddy*)((IMUserDaemon*)usersSearch->data)->dowry;
					
					usersSearch = usersSearch->next;
				}
				
				delete (IMBuddy*)owner->dowry;
				
				self->window->removeStatusButton((IMOwnerDaemon*)data);
				
				// if there is only one owner active don't show any buttons
				if (g_list_length(IO_getOwnerList()) == 1)
					self->window->removeStatusButton((IMOwnerDaemon*)IO_getOwnerList()->data);
			}
	
			// reload all groups
			IO_getGroupManager()->checkForNewGroups();
	
			ownerSearch = IO_getOwnerList();
			while(ownerSearch)
			{
				usersSearch = ((IMOwnerDaemon*)ownerSearch->data)->users;
				while(usersSearch)
				{
					((IMUserDaemon*)usersSearch->data)->editUserInformation(TRUE, USER_UPDATE_BASIC);
					usersSearch = usersSearch->next;
				}
				ownerSearch = ownerSearch->next;
			}

			// reload the contact list
			self->rebuildContactList(FALSE);
	
			// rebuild the window title
			self->window->setWindowTitle(self->eventCount || self->ownerEventCount);
	
			
			// enable / disable the buttons
			self->window->setLockUserMenus(IO_getOwnerList() == NULL);
			
			break;

			
	}	

	return TRUE;	
}


userEventType IMOwner::getWindowIDForEvent(gint eventType)
{	
	switch(eventType)
	{
		case -1:
		case USER_EVENT_MESSAGE:
			return UE_MESSAGE;
				
		case USER_EVENT_FILE:
			return UE_FILETRANSFER;
				
		case USER_EVENT_CHAT:
			return UE_CHAT;
				
		case USER_EVENT_URL:
			return UE_URL;
				
		case USER_EVENT_AUTH_REQUEST:
			return UE_AUTHORIZATION;
		
		case USER_EVENT_AUTH_ACCEPT:
			return UE_AUTHORIZATIONGRANT;
		
		case USER_EVENT_AUTH_REFUSE:
			return UE_AUTHORIZATIONREFUSE;
		
		case USER_EVENT_ADDED_TO_LIST:
			return UE_ADDEDTOLIST;
		
		case USER_EVENT_CONTACTS:
			return UE_CONTACTS;
		
		case USER_EVENT_WEBPANEL:
			return UE_WEBPANEL;
		
		case USER_EVENT_PAGER:
			return UE_PAGER;
		
		case USER_EVENT_SERVERMSG:
			return UE_SERVERMSG;
				
		default:
			fprintf(stderr, "IMOwner::getWindowIDForEvent: Unknown event type (%d)!\n", eventType);
			return UE_MESSAGE;
	}		
}
