#include "settings.h"
#include "imuserdaemon.h"


settings::settings(gchar *filename)
{
	gchar *path;
	
	
	sections = NULL;
	
	licqFile = new CIniFile(INI_FxALLOWxCREATE);	
	
	// open config file
	path = g_strdup_printf("%s/%s", BASE_DIR, filename);	
	licqFile->LoadFile(path);
	g_free(path);
						  	
	
	// get the custom text properties
	loadTextTags();
	
	// also we install a listener for the text tags
	installListener("appearance", (settingChangedFunc)this->cb_textTagsChangedCallback, this, TRUE);
}



settings::~settings()
{
	licqFile->FlushFile();
	licqFile->CloseFile();
	// FIXME: remove all setting elements
}



void settings::installProperties(gchar* sname, ...)
{
	GList 			*search;
	settingsSection *section;
	settingsProperty *prop;
	va_list 		arguments;
	gchar			*name,
					*tmp;
	GType			type;
	GValue			*val;
	
	
	// find the section
	search = sections;
	while(search)
	{
		section = (settingsSection*)search->data;
		
		if (!strcmp(section->name, sname))
			break;
		
		search = search->next;
	}		
	
	// section was not found -> create a new
	if (!search)
	{
		section = (settingsSection*)g_malloc0(sizeof(settingsSection));
		section->name = g_strdup(sname);
		sections = g_list_append(sections, section);
	}
	
	
	licqFile->SetSection(sname);
	
	va_start(arguments, sname);
	
	
	// check whether the properties exist and install them if they don't
	while((name = va_arg(arguments, gchar*)) && (type = va_arg(arguments, GType)))
	{
		search = section->properties;
		while(search)
		{
			prop = (settingsProperty*)search->data;
			
			// if this property exists don't search for it any longer
			if (G_VALUE_TYPE(prop->val) == type && !strcmp(prop->name, name))
				break;
			
			search = search->next;
		}
		
		// the property doesn't exist yet -> create it
		if (!search)
		{
			val = (GValue*)g_malloc0(sizeof(GValue));
			val = g_value_init(val, type); 
			
			switch(type)
			{
				case G_TYPE_ULONG:
					gulong toSetInt, defaultInt;
				
					defaultInt = va_arg(arguments, gulong);
					licqFile->ReadNum(name, toSetInt, defaultInt);
					g_value_set_ulong(val, toSetInt);
					break; 
				
				case G_TYPE_STRING:
					gchar	*toSetString,
							*defaultString;				
					defaultString = va_arg(arguments, gchar*);
					toSetString = (gchar*)g_malloc0(1024);
				
					licqFile->ReadStr(name, toSetString, defaultString);
					g_value_set_string(val, toSetString);
					break;
				
				case G_TYPE_BOOLEAN:
					bool toSetBool, defaultBool;
				
					defaultBool = va_arg(arguments, gboolean);
				
					licqFile->ReadBool(name, toSetBool, defaultBool);
					g_value_set_boolean(val, toSetBool != 0);
					break;
				
				
				case ICQND_TYPE_COLOR:
					GdkColor *col1, *col2;
					GdkColor dummy;
					gchar	 *txt;
				
					col1 = (GdkColor*)va_arg(arguments, gpointer);
				
					if (!col1)
						col1 = &dummy;
					
					col2 = (GdkColor*)g_malloc0(sizeof(GdkColor));
				
					txt = g_strdup_printf("%sred", name);
					licqFile->ReadNum(txt, (gulong&)col2->red, col1->red);					
					g_free(txt);

					txt = g_strdup_printf("%sgreen", name);
					licqFile->ReadNum(txt, (gulong&)col2->green, col1->green);					
					g_free(txt);

					txt = g_strdup_printf("%sblue", name);
					licqFile->ReadNum(txt, (gulong&)col2->blue, col1->blue);					
					g_free(txt);				
					
					g_value_set_pointer(val, col2);
					break;
					
					
				default:
					fprintf(stderr, "settings::installProperties: Unknown variable type to initialize!\n");
					return;
			}
			
			prop = (settingsProperty*)g_malloc0(sizeof(settingsProperty));
			prop->name = g_strdup(name);
			prop->val = val;
			section->properties = g_list_append(section->properties, prop);
		}	
	}
}
	



void settings::setProperties(gboolean inform, gchar* sname, ...)
{
	GList 			*search,
					*searchhatches,
					*properties2,
					*informProps;
	settingsSection 	*section;
	settingsProperty *prop;
	settingsFunc	*func;
	va_list 		arguments;
	gchar			*name;
	gboolean		propChanged;

	
	// find the section
	search = sections;
	while(search)
	{
		section = (settingsSection*)search->data;
		
		if (!strcmp(section->name, sname))
			break;
		
		search = search->next;
	}	

	if (!search)
		return;		
		
	
	
	va_start(arguments, sname);

	properties2 = g_list_copy(section->properties);
	informProps = NULL;
	
	
	// check whether the properties exist
	while((name = va_arg(arguments, gchar*)))
	{	
		search = properties2;
		while(search)
		{
			prop = (settingsProperty*)search->data;
			
			// if this property exists don't search for it any longer
			if (!strcmp(prop->name, name))
			{
				propChanged = FALSE;
				
				switch(G_VALUE_TYPE(prop->val))
				{
					case G_TYPE_STRING:
						gchar	*toSetString,
								*oldString;
					
						toSetString = va_arg(arguments, gchar*);
						oldString = (gchar*)g_value_get_string(prop->val);
						
						if (strcmp(toSetString, oldString)!=0)
						{
							g_value_set_string(prop->val, toSetString);
							propChanged = TRUE;
						}
						break;
					
						
					case G_TYPE_ULONG:
						gulong 	toSetLong,
								oldLong;
					
						toSetLong = va_arg(arguments, gulong);
						oldLong = g_value_get_ulong(prop->val);
					
						if (toSetLong != oldLong)
						{
							g_value_set_ulong(prop->val, toSetLong);
							propChanged = TRUE;
						}
						break;
					
					case G_TYPE_BOOLEAN:
						gboolean toSetBool,
								 oldBool;
					
						toSetBool = va_arg(arguments, gboolean);
						oldBool	  = g_value_get_boolean(prop->val);
					
						if (toSetBool != oldBool)
						{
							g_value_set_boolean(prop->val, toSetBool);
							propChanged = TRUE;
						}
						break;
					
					case G_TYPE_POINTER:
						GdkColor	*col1, 
									*col2;
					
						col1 = va_arg(arguments, GdkColor*);
						col2 = (GdkColor*)g_value_get_pointer(prop->val);
					
					
						if (col1->red != col2->red || 
							col1->green != col2->green || 
							col1->blue != col2->blue)
						{									
							memcpy(col2, col1, sizeof(GdkColor));
							propChanged = TRUE;
						}
						break;
						
						
					default:
						fprintf(stderr, "settings::setProperties: Unknown property to change\n");
						break;
				}
				
				// apply the hatches
				if (propChanged)
				for (searchhatches = prop->hatches; searchhatches; searchhatches = searchhatches->next)
					applyPropertyToHatch(prop->val, searchhatches->data);
				
				
				// FIXME: don't add properties to the changed list that didn't change their value
				if (inform && propChanged)
					informProps = g_list_append(informProps, prop);
				
				
				// remove this element from the copied list, so it won't be browsed again
				properties2 = g_list_remove(properties2, search->data);
				break;
			}
			
			search = search->next;
		}
		
		// nothing was found -> stop searching
		if (!search)
		{
			fprintf(stderr, "settings::setProperties: Property %s could not be found\n", name);
			break;
		}	
	}
	
	
	// some properties changed -> call their functions
	if (informProps)
	{
		search = section->funcs;
		while(search)
		{
			func = (settingsFunc*)search->data;
			
			func->func(sname, informProps, func->data);
			search = search->next;
		}
			
		g_list_free(informProps);
	}
	
	g_list_free(properties2);
	
}


	
void settings::getProperties(gchar* sname, ...)
{
	GList 			*search,
					*properties2;
	settingsSection 	*section;
	settingsProperty *prop;
	va_list 		arguments;
	gchar			*name;
	
	// find the section
	search = sections;
	while(search)
	{
		section = (settingsSection*)search->data;
		
		if (!strcmp(section->name, sname))
			break;
		
		search = search->next;
	}	
	
	if (!search)
		return;		
		
	
	va_start(arguments, sname);
	
	properties2 = g_list_copy(section->properties);
	
	// check whether the properties exist
	while((name = va_arg(arguments, gchar*)))
	{	
		search = properties2;
		while(search)
		{
			prop = (settingsProperty*)search->data;		
			
			// if this property exists don't search for it any longer
			if (!strcmp(prop->name, name))
			{
				switch(G_VALUE_TYPE(prop->val))
				{
					case G_TYPE_ULONG:
						*va_arg(arguments, gulong*) = g_value_get_ulong(prop->val);
						break;
					
					case G_TYPE_STRING:
						*va_arg(arguments, gchar**) = g_strdup((gchar*)g_value_get_string(prop->val));
						break;
						
					case G_TYPE_BOOLEAN:
						*va_arg(arguments, gboolean*) = g_value_get_boolean(prop->val);
						break;
						
					case G_TYPE_POINTER:
						memcpy(va_arg(arguments, GdkColor*), g_value_get_pointer(prop->val), sizeof(GdkColor));
						break;
						
					default:
						fprintf(stderr, "settings::getProperties: invalid property type!\n");
						return;
				} 	

				// remove this element from the copied list, so it won't be browsed again
				properties2 = g_list_remove(properties2, search->data);
				break;
			}
			
			search = search->next;
		}
		
		// nothing was found -> stop searching
		if (!search)
		{
			fprintf(stderr, "settings::getProperties: Property %s could not be found\n", name);
			break;
		}	
	}	
	
	g_list_free(properties2);
}
	


void settings::installListener(gchar* sname, settingChangedFunc func, gpointer data, gboolean prepend)
{
	GList 			*search;
	settingsSection *section;
	settingsFunc	*newFunc;
	
	// find the section
	search = sections;
	while(search)
	{
		section = (settingsSection*)search->data;
		
		if (!strcmp(section->name, sname))
			break;
		
		search = search->next;
	}	
	
	if (!search)
		return;		

	newFunc = (settingsFunc*)g_malloc0(sizeof(settingsFunc));

	newFunc->func = func;
	newFunc->data = data;

	if (prepend)
		section->funcs = g_list_prepend(section->funcs, newFunc);
	else
		section->funcs = g_list_append(section->funcs, newFunc);	
}
	



void settings::removeListenerByData(gpointer data)
{
	GList 			*search,
					*searchf;
	settingsSection *section;
	settingsFunc	*func;

	search = sections;

	while(search)
	{
		section = (settingsSection*)search->data;
		searchf = section->funcs;
		
		while(searchf)
		{
			func = (settingsFunc*)searchf->data;
			
			if (func->data == data)
			{
				section->funcs = g_list_remove(section->funcs, func);
				g_free(func);
				break;
			}
			
			searchf = searchf->next;
		}
		
		search = search->next;
	}		
}




void settings::loadTextTags()
{
	GtkTextTag		*atag;
	GdkColor		colRed,
					colBlue,
					colHRed,
					colHBlue;
	GtkWidget		*tmpW;
	
	// predefine colors
	GDK_COLOR_SET(&colRed, 0xAD00, 0x2C00, 0x2900);
	GDK_COLOR_SET(&colBlue, 0x1000, 0x5500, 0x9C00);
	
	// set the average between the text area's background and our normal colors
	tmpW = gtk_text_view_new();
	gtk_widget_ensure_style(tmpW);
	GDK_COLOR_SET_AV(&colHRed, &colRed, &tmpW->style->base[GTK_STATE_NORMAL]);
	GDK_COLOR_SET_AV(&colHBlue, &colBlue, &tmpW->style->base[GTK_STATE_NORMAL]);
	gtk_widget_destroy(tmpW);
	
	
	// install their properties and set defaults if there are no values
	installProperties("appearance",
					  "colorReceive", ICQND_TYPE_COLOR, &colRed,
					  "colorSend"	, ICQND_TYPE_COLOR, &colBlue,
					  "colorHReceive", ICQND_TYPE_COLOR, &colHRed,
					  "colorHSend", ICQND_TYPE_COLOR, &colHBlue,
					  NULL);
					  
	// read the colors from the config file
	getProperties("appearance",
				  "colorReceive", &colRed,
				  "colorSend", &colBlue,
				  "colorHReceive", &colHRed,
				  "colorHSend", &colHBlue,
				  NULL);
	
		
	textTags = gtk_text_tag_table_new();


	// incoming messages
	atag = gtk_text_tag_new("incoming");
	g_object_set(atag, "foreground-gdk", &colRed,
					   NULL);
	gtk_text_tag_table_add(textTags, atag);
	
	//outgoing messages
	atag = gtk_text_tag_new("outgoing");
	g_object_set(atag, "foreground-gdk", &colBlue, 
						NULL);
	gtk_text_tag_table_add(textTags, atag);	
	
	// errors
	atag = gtk_text_tag_new("error");
	g_object_set(atag, 	"style", PANGO_STYLE_ITALIC, NULL);
	gtk_text_tag_table_add(textTags, atag);
	
	
	// history receive
	atag = gtk_text_tag_new("history_incoming");
	g_object_set(atag, "foreground-gdk", &colHRed, NULL);
	gtk_text_tag_table_add(textTags, atag);
      
	  
	// history send
	atag = gtk_text_tag_new("history_outgoing");
	g_object_set(atag, "foreground-gdk", &colHBlue, NULL);
	gtk_text_tag_table_add(textTags, atag);
	
	
	//notification
	atag = gtk_text_tag_new("notification");
	g_object_set(atag, "weight", PANGO_WEIGHT_BOLD, NULL);
	gtk_text_tag_table_add(textTags, atag);
	
	// url
	atag = gtk_text_tag_new("url");
	g_object_set(atag, "underline", PANGO_UNDERLINE_SINGLE,
					   NULL);
	gtk_text_tag_table_add(textTags, atag);
	
	// headlines
	atag = gtk_text_tag_new("HL");
	g_object_set(atag, 	"weight", PANGO_WEIGHT_BOLD, NULL);
	gtk_text_tag_table_add(textTags, atag);		
	
	// new line space
	atag = gtk_text_tag_new("newline");
	g_object_set(atag, 	"size", 3*PANGO_SCALE, NULL);
	gtk_text_tag_table_add(textTags, atag);	

	// margin
	atag = gtk_text_tag_new("margin");
	g_object_set(atag, 	"left_margin", 20, NULL);
	gtk_text_tag_table_add(textTags, atag);		

	// timestamp
	atag = gtk_text_tag_new("timestamp");
	g_object_set(atag,  "size", 8*PANGO_SCALE,
						"weight", PANGO_WEIGHT_LIGHT,
						NULL);
	gtk_text_tag_table_add(textTags, atag);	
}



void settings::installHatch(gchar* sname, gchar* propname, gpointer hatch)
{
	GList 				*search;
	settingsSection 	*section;
	settingsProperty	*prop;
	
	// find the section
	for(search = sections; search; search = search->next)
	{
		section = (settingsSection*)search->data;
		
		if (!strcmp(section->name, sname))
			break;
	}
		
	if (!search)
		return;		
	
	// find the property
	for (search = section->properties; search; search = search->next)
	{
		prop = (settingsProperty*)search->data;

		if (!strcmp(prop->name, propname))
			break;
	}
	
	if (!search)
		return;
	
	
	prop->hatches = g_list_append(prop->hatches, hatch);

	// init the value	
	applyPropertyToHatch(prop->val, hatch);
}
	

void settings::removeHatch(gchar* sname, gchar* propname, gpointer hatch)
{
	GList 				*search;
	settingsSection 	*section;
	settingsProperty	*prop;
	
	// find the section
	for(search = sections; search; search = search->next)
	{
		section = (settingsSection*)search->data;
		
		if (!strcmp(section->name, sname))
			break;
	}
		
	if (!search)
		return;		

	// find the property
	for (search = section->properties; search; search = search->next)
	{
		prop = (settingsProperty*)search->data;
		
		if (!strcmp(prop->name, propname))
			break;
	}
	
	if (!search)
		return;
	
	prop->hatches = g_list_remove(prop->hatches, hatch);	
}


void settings::applyPropertyToHatch(GValue *prop, gpointer hatch)
{	
	switch(G_VALUE_TYPE(prop))
	{
		case G_TYPE_STRING:
			gchar	**hstr;
			hstr = (gchar**)hatch;
			if (*hstr)
				g_free(*hstr);
			*hstr = g_strdup(g_value_get_string(prop));
			break;
			
		case G_TYPE_ULONG:
			*((gulong*)hatch) = g_value_get_ulong(prop);
			break;
		
		case G_TYPE_BOOLEAN:
			*((gboolean*)hatch) = g_value_get_boolean(prop);
			break;
		
		case G_TYPE_POINTER:
			GdkColor	*col1,
						*col2;
			col1 = (GdkColor*)hatch;
			col2 = (GdkColor*)g_value_get_pointer(prop);
			col1->red = col2->red;
			col1->green = col2->green;
			col1->blue = col2->blue;
			break;
	}		
	
}




gboolean settings::writeSettings()
{
	GList				*searchs,
						*searchp;
	settingsSection		*section;
	settingsProperty 	*property;
	
	searchs = sections;
	
	while(searchs)
	{
		section = (settingsSection*)searchs->data;
		if (!licqFile->SetSection(section->name))
			licqFile->CreateSection(section->name);

		searchp = section->properties;
		while(searchp)
		{
			property = (settingsProperty*)searchp->data;
			
			switch(G_VALUE_TYPE(property->val))
			{
				case G_TYPE_ULONG:
					licqFile->WriteNum(property->name, g_value_get_ulong(property->val));
					break;
				
				case G_TYPE_STRING:
					licqFile->WriteStr(property->name, g_value_get_string(property->val));
					break;
				
				case G_TYPE_BOOLEAN:
					licqFile->WriteBool(property->name, g_value_get_boolean(property->val));
					break;
				
				case G_TYPE_POINTER:
					GdkColor *col;
					gchar	 *txt;
					col = (GdkColor*)g_value_get_pointer(property->val);
					if (!col)
						break;
														
					txt = g_strdup_printf("%sred", property->name);
					licqFile->WriteNum(txt, (gulong)col->red);
					g_free(txt);

					txt = g_strdup_printf("%sgreen", property->name);
					licqFile->WriteNum(txt, (gulong)col->green);
					g_free(txt);

					txt = g_strdup_printf("%sblue", property->name);
					licqFile->WriteNum(txt, col->blue);
					g_free(txt);
					break;					
			}
			
			searchp = searchp->next;
		}			
		
		
		searchs = searchs->next;
	}
	
	return TRUE;
}



void settings::cb_textTagsChangedCallback(gchar* sname, GList *values, settings* self)
{
	GList				*search;
	settingsProperty	*prop;
	GtkWidget			*tmpW;
	GdkColor			*col,
						hcol;
	GtkTextTag			*tag;
	
	search = values;
	
	tmpW = gtk_text_view_new();
	gtk_widget_ensure_style(tmpW);
	
	while(search)
	{
		prop = (settingsProperty*)search->data;
		
		// the receiving color was changed
		if (!strcmp(prop->name, "colorReceive"))
		{
			col = (GdkColor*)g_value_get_pointer(prop->val);
			GDK_COLOR_SET_AV(&hcol, col, &tmpW->style->base[GTK_STATE_NORMAL]);
			tag = gtk_text_tag_table_lookup(self->textTags, "incoming");
			g_object_set(G_OBJECT(tag), "foreground-gdk", col, NULL);

			tag = gtk_text_tag_table_lookup(self->textTags, "history_incoming");
			g_object_set(G_OBJECT(tag), "foreground-gdk", &hcol, NULL);	
		} else
		if (!strcmp(prop->name, "colorSend"))
		{
			col = (GdkColor*)g_value_get_pointer(prop->val);
			GDK_COLOR_SET_AV(&hcol, col, &tmpW->style->base[GTK_STATE_NORMAL]);
			tag = gtk_text_tag_table_lookup(self->textTags, "outgoing");
			g_object_set(G_OBJECT(tag), "foreground-gdk", col, NULL);

			tag = gtk_text_tag_table_lookup(self->textTags, "history_outgoing");
			g_object_set(G_OBJECT(tag), "foreground-gdk", &hcol, NULL);				
		}
		
		search = search->next;
	}
	
	gtk_widget_destroy(tmpW);	
}




settings* settings_d;

settings* settings_getSettings()
{
	return settings_d;
}


void	  settings_setSettings(settings* s)
{
	settings_d = s;	
}
