/***************************************************************************
 *            utilities.cpp
 *
 *  Thu Aug 26 01:24:06 2004
 *  Copyright  2004  User
 *  Email
 ****************************************************************************/

#include "utilities.h"

#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <gdk/gdkx.h>

GdkCursor	*u_cursorHover = NULL;
GdkCursor	*u_cursorNormal = NULL;


u_browserInfo* u_getBrowserByCommand(gchar* cmd)
{
	guint i;
	
	for (i=0; icqndWebBrowsers[i].name; i++)
		if (!strcmp(cmd, icqndWebBrowsers[i].command))
			return (u_browserInfo*)&icqndWebBrowsers[i];
		
	return NULL;
}


u_browserInfo* u_getFirstInstalledBrowser()
{
	guint 	i;
	gchar	*cmd;
	
	for (i=0; icqndWebBrowsers[i].name; i++)
	{
		cmd = g_find_program_in_path(icqndWebBrowsers[i].command);
		if (cmd)
		{
			g_free(cmd);
			return (u_browserInfo*)&icqndWebBrowsers[i];
		}
	}
	
	return NULL;
}


GList* u_getInstalledBrowsers()
{
	guint 	i;
	GList	*installed;
	gchar	*program;
	
	i = 0;
	installed = NULL;
	
	// find all programs that are installed
	while(icqndWebBrowsers[i].name)
	{
		program = g_find_program_in_path(icqndWebBrowsers[i].command);
		if (program)
		{
			installed = g_list_append(installed, (void*)&icqndWebBrowsers[i]);
			g_free(program);
		}
		
		i++;
	}
	
	return installed;
}



void u_showAlertMessage(gchar* headline, gchar *message, gchar* iconstock) 
{

	GtkWidget 	*dialog, 
   				*label,
				*hbox,
				*icon;
										 
	label = gtk_label_new(message);
	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
	gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
	gtk_widget_set_size_request(label, 300, -1);
	
	icon = gtk_image_new_from_stock(iconstock, GTK_ICON_SIZE_DIALOG);
	gtk_misc_set_alignment(GTK_MISC(icon), 0.5f, 0.0f);
   
	hbox = gtk_hbox_new(FALSE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(hbox), 10);
	gtk_box_pack_start(GTK_BOX(hbox), icon, FALSE, FALSE, 5);
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 5);

   


	dialog = gtk_dialog_new_with_buttons (	headline,
											NULL,
											GTK_DIALOG_MODAL,
											GTK_STOCK_OK,
											GTK_RESPONSE_NONE,
											NULL);
								
	g_signal_connect(dialog, "response", G_CALLBACK(gtk_widget_destroy), NULL);								


	gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), hbox);

	gtk_widget_show_all(dialog);
}




gint u_showYesNoDialog(gchar* headline, gchar *message, gchar* iconstock, gboolean hasCancel) 
{

	GtkWidget 	*dialog, 
   				*label,
				*hbox,
				*icon;
	gint		res;
										 
	label = gtk_label_new(message);
	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
	gtk_widget_set_size_request(label, 200, -1);
	icon = gtk_image_new_from_stock(iconstock, GTK_ICON_SIZE_DIALOG);
   	gtk_misc_set_alignment(GTK_MISC(icon), 0.5f, 0.0f);
	
	hbox = gtk_hbox_new(FALSE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
	gtk_box_pack_start(GTK_BOX(hbox), icon, FALSE, FALSE, 5);
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 5);

	

	if (hasCancel)	
		dialog = gtk_dialog_new_with_buttons (	headline,
												NULL,
												GTK_DIALOG_MODAL,
												GTK_STOCK_YES,
												GTK_RESPONSE_YES,
												GTK_STOCK_NO,
												GTK_RESPONSE_NO,
												GTK_STOCK_CANCEL,
												GTK_RESPONSE_CANCEL,
												NULL);
	else
		dialog = gtk_dialog_new_with_buttons (	headline,
												NULL,
												GTK_DIALOG_MODAL,
												GTK_STOCK_YES,
												GTK_RESPONSE_YES,
												GTK_STOCK_NO,
												GTK_RESPONSE_NO,
												NULL);	

	gtk_window_set_destroy_with_parent(GTK_WINDOW(dialog), TRUE);	
	
	gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), hbox);
	gtk_widget_show_all(hbox);

	res = gtk_dialog_run (GTK_DIALOG(dialog));
	
	gtk_widget_destroy(dialog);
	
	return res;
}



gint u_showYNRememberDialog(gint dialogID, gchar* headline, GtkWidget *content, ...)
{
	va_list 	arguments;
	
	GtkWidget 	*dialog, 
   				*askAgainCheckbox,
				*but,
				*hbox,
				*vbox,
				*icon;
	
	gint		res, curID;	
	gpointer	imgP, txtP;
	
	
	// FIXME: check settings value (and return this if there is one set)
	
	
	// create the icon
	icon = gtk_image_new_from_stock(GTK_STOCK_DIALOG_QUESTION, GTK_ICON_SIZE_DIALOG);
	gtk_misc_set_alignment(GTK_MISC(icon), 0.5f, 0.0f);
   
	// create the check box
	askAgainCheckbox = gtk_check_button_new_with_label(tr("Do not ask again!"));
	
	// add icon + content widget into a hbox
	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), icon, FALSE, FALSE, 5);
	gtk_box_pack_start(GTK_BOX(hbox), content, TRUE, TRUE, 5);
	
	// add hbox + ask again field into a vbox
	vbox = gtk_vbox_new(FALSE,6);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), askAgainCheckbox, FALSE, TRUE, 0);


	dialog = gtk_dialog_new_with_buttons (	headline,
											NULL,
											GTK_DIALOG_MODAL,
											NULL);
											
	
	// add all arguments (order is: text, stockID ...)
	va_start(arguments, content);
	curID = 1;
	
	while((txtP = va_arg(arguments, gpointer)) && (imgP = va_arg(arguments, gpointer)))
	{
		but = u_createImageButton(gtk_label_new((gchar*)txtP), 
								  gtk_image_new_from_stock((gchar*)imgP, GTK_ICON_SIZE_BUTTON)
								 );
		gtk_dialog_add_action_widget(GTK_DIALOG(dialog), 
									 but,		
									 curID);
		
		gtk_widget_show_all(but);
		curID ++;
	}
	
	
	// add everything and run the dialog
	gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), vbox);
	gtk_widget_show_all(vbox);

	res = gtk_dialog_run (GTK_DIALOG(dialog));
	
	// update settings value
	
	
	gtk_widget_destroy(dialog);
	
	return res;
	
}




void u_showFileWriteError(FILE* f)
{
	u_showAlertMessage(tr("Write error"), tr("The file could be created\nBut not filled.\nIs there no disk space available?"), GTK_STOCK_DIALOG_ERROR);
	fclose(f);
}



GtkWidget* u_createStockImageButton(gchar* stockIcon)
{
	GtkWidget	*btn,
				*img;
	
	img = gtk_image_new_from_stock(stockIcon, GTK_ICON_SIZE_SMALL_TOOLBAR);
	btn = gtk_button_new();
	gtk_button_set_relief(GTK_BUTTON(btn), GTK_RELIEF_HALF);
	gtk_container_add(GTK_CONTAINER(btn), img);
	
	return btn;
}



void u_setStockButtonImage(GtkWidget* button, gchar* stockIcon)
{
	GList	*children,
			*search;
	
	children = gtk_container_get_children(GTK_CONTAINER(button));
	
	search = children;
	while(search)
	{
		if (GTK_IS_IMAGE(search->data))
		{
			gtk_image_set_from_stock(GTK_IMAGE(search->data), stockIcon, GTK_ICON_SIZE_SMALL_TOOLBAR);
			break;
		}
		search = search->next;
	}
	
	g_list_free(children);
}


GtkWidget* u_createTextStockImageButton(gchar* text, gchar* stockIcon)
{
	GtkWidget	*btn,
				*img,
				*hbox;
	
	
	
	img = gtk_image_new_from_stock(stockIcon, GTK_ICON_SIZE_BUTTON);
	hbox = gtk_hbox_new(FALSE, 3);
	gtk_box_pack_start(GTK_BOX(hbox), img, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new_with_mnemonic(text), TRUE, TRUE, 0);	
	btn = gtk_button_new();
	gtk_container_add(GTK_CONTAINER(btn), hbox);
	
	return btn;
}



GtkWidget* u_changeTextStockImageButton(GtkWidget* w, gchar* text, gchar* stockIcon)
{
	GList	*children,
			*search;
	GtkWidget	*img,
				*hbox;
	
	children = gtk_container_get_children(GTK_CONTAINER(w));
	search = children;
	while(search)
	{
		gtk_container_remove(GTK_CONTAINER(w), (GtkWidget*)search->data);
		search = search->next;
	}
	
	img = gtk_image_new_from_stock(stockIcon, GTK_ICON_SIZE_BUTTON);
	hbox = gtk_hbox_new(FALSE, 3);
	gtk_box_pack_start(GTK_BOX(hbox), img, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new_with_mnemonic(text), TRUE, TRUE, 0);
	
	gtk_container_add(GTK_CONTAINER(w), hbox);
	gtk_widget_show_all(w);

	return w;	
}



GtkWidget* u_createImageButton(GtkWidget* label, GtkWidget* img)
{
	GtkWidget   *btnbox,
				*btn;

	gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
	
	btnbox = gtk_hbox_new(FALSE, 3);

	gtk_box_pack_start(GTK_BOX(btnbox), img, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(btnbox), label, TRUE, TRUE, 0);
	
	btn = gtk_button_new();		
	
	gtk_container_add(GTK_CONTAINER(btn), btnbox);	
	
	return btn;	 
}



void u_setWidgetBackground(GtkWidget *widget, gchar* col)
{
	GdkColor  b;
	GtkStyle  *newstyle;
	
	gdk_color_parse (col, &b);

	newstyle = gtk_style_copy (gtk_widget_get_style (widget));
	int i;
	for ( i = 0; i < 5; i++ )
		newstyle->bg[i] = b;
	gtk_widget_set_style (widget, newstyle);
}




GtkWidget* u_createTextView(GtkWidget** tv)
{
	GtkWidget  *scroll;

	*tv = gtk_text_view_new();
	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(*tv), GTK_WRAP_WORD_CHAR);
	
	
	scroll = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_container_add(GTK_CONTAINER(scroll), *tv);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scroll), GTK_SHADOW_IN);
		
	return scroll;
}



void u_createTableRow(GtkWidget* tbl, char* name1, GtkWidget** el1, char* name2, GtkWidget** el2, gint row)
{
	GtkWidget* label;
	
	label = gtk_label_new(name1);
	gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
	*el1 = gtk_entry_new();
	gtk_editable_set_editable(GTK_EDITABLE(*el1), FALSE);
	gtk_widget_set_size_request(*el1, 100, -1);
	
	gtk_table_attach_defaults(GTK_TABLE(tbl), label, 0, 1, row, row+1);
	gtk_table_attach_defaults(GTK_TABLE(tbl), *el1, 1, 2, row, row+1);	


	label = gtk_label_new(name2);
	gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
	*el2 = gtk_entry_new();
	gtk_editable_set_editable(GTK_EDITABLE(*el2), FALSE);
	gtk_widget_set_size_request(*el2, 100, -1);
	
	gtk_table_attach_defaults(GTK_TABLE(tbl), label, 2, 3, row, row+1);
	gtk_table_attach_defaults(GTK_TABLE(tbl), *el2, 3, 4, row, row+1);		
}




GtkWidget* u_createHeadline(gchar* txt, gchar* alttxt, gchar* imageStock)
{
	GtkWidget	*hlimg,
				*hllabel,
				*hlbox,
				*hlev,
				*hlframe;
	
	gchar		*hl;

	
	hl = g_strdup_printf(
				"<span size=\"x-large\" weight=\"bold\">%s%s</span>",
				txt, alttxt);
	
	hlimg = gtk_image_new_from_stock(imageStock, GTK_ICON_SIZE_LARGE_TOOLBAR);
	hllabel = gtk_label_new("");
	gtk_label_set_markup(GTK_LABEL(hllabel), hl);
	g_free(hl);

	
	hlbox = gtk_hbox_new(FALSE,10);	
	gtk_box_pack_start(GTK_BOX(hlbox), hlimg, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hlbox), hllabel, FALSE, TRUE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(hlbox), 3); 
	
	hlev = gtk_event_box_new();
	gtk_widget_set_state(hlev, GTK_STATE_PRELIGHT);
	gtk_container_add(GTK_CONTAINER(hlev), hlbox);

	hlframe = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(hlframe), GTK_SHADOW_OUT);
	gtk_container_add(GTK_CONTAINER(hlframe), hlev);
	
	return hlframe;
}




gchar* u_convertSize2Readable(gint size)
{
	gint unit;
	float sizec;
	
	unit = 0;
	sizec = size;
	
	while(sizec>=1024)
	{
		sizec = sizec / 1024.0f;
		unit++;
	}
	
	
	switch(unit)
	{
		case 0: return g_strdup_printf("%3.1f B", sizec);
		case 1: return g_strdup_printf("%3.1f KB", sizec);
		case 2: return g_strdup_printf("%3.1f MB", sizec);
		case 3: return g_strdup_printf("%3.1f GB", sizec);
		default: return g_strdup("");
	}
}



gchar* u_getTimeDistance(time_t t1, time_t t2)
{
	time_t	newTime;
	GString	*timeStr;
	gulong	tmpTime;
	
	newTime = t1 - t2;
	timeStr = g_string_new("");
			
	// get the days
	tmpTime = newTime / (60*60*24);
	if (tmpTime)			
		g_string_append_printf(timeStr, tr("%ldd"), tmpTime);
	newTime-=tmpTime*(60*60*24);
			
	// get the hours
	tmpTime = newTime / (60*60);
	if (tmpTime)
	{
		if (*timeStr->str)
			timeStr = g_string_append(timeStr, " ");
		g_string_append_printf(timeStr, tr("%ldh"), tmpTime);
	}
	newTime-=tmpTime*(60*60);
			
	// get the minutes
	tmpTime = newTime / 60;
	if (tmpTime)
	{
		if (*timeStr->str)
			timeStr = g_string_append(timeStr, " ");
		g_string_append_printf(timeStr, tr("%ldm"), tmpTime);
	}
	newTime-=tmpTime*60;
			
	// and the seconds
	if (*timeStr->str)
		timeStr = g_string_append(timeStr, " ");
	g_string_append_printf(timeStr, tr("%lds"), newTime);
	
	return g_string_free(timeStr, FALSE);
}




GtkMenu* u_createMenuList(u_menuItem* menulist, gchar elements, GtkSignalFunc cbfunc, gint menuID, gpointer data, gboolean showIcons)
{
	GtkWidget* 	 	menu;
	GtkWidget*   	item;
	gint		 	i;
	GtkWidget*	 	hBox;
	GtkWidget*		icon;
	GtkWidget*	 	title;
	GtkWidget*		frame;
	GSList*			radioGroups[32] = {NULL,};
	
	item = NULL;
	title = NULL;
	icon = NULL;
	menu = gtk_menu_new();
		
	for (i=0;i<elements;i++)
	{	
		switch(menulist[i].type)
		{
			// a separator
			case UMENU_SEPARATOR:
				item = gtk_separator_menu_item_new();
				break;
			
			// a headline
			case UMENU_HEADLINE:
				title = gtk_label_new(tr(menulist[i].title));
				gtk_misc_set_alignment(GTK_MISC(title), 0.0f, 0.5f);
			
				frame = gtk_frame_new(NULL);
				gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_OUT);
				gtk_container_add(GTK_CONTAINER(frame), title);
			
				item = gtk_menu_item_new();
				gtk_container_add(GTK_CONTAINER(item), frame);
				break;
			
			// a normal menu item
			case UMENU_NORMAL:
				
				//item  = gtk_image_menu_item_new_with_mnemonic(tr(menulist[i].title));
		
				title = gtk_label_new_with_mnemonic(menulist[i].title);
				gtk_misc_set_alignment(GTK_MISC(title), 0.0f, 0.5f);
			
				if (showIcons)
				{
					if (menulist[i].pixbuf)
						icon = gtk_image_new_from_pixbuf(menulist[i].pixbuf);
					else
						icon = gtk_image_new_from_stock(menulist[i].stockIcon, GTK_ICON_SIZE_MENU);
					gtk_widget_set_size_request(icon, 16, 16);
				}
				//gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), icon); */
				hBox = gtk_hbox_new(FALSE, 2);
				if (showIcons)
					gtk_box_pack_start(GTK_BOX(hBox), icon, FALSE, FALSE, 2);
				gtk_box_pack_start(GTK_BOX(hBox), title, TRUE, TRUE, 0);
			
				item = gtk_menu_item_new();
				gtk_container_add(GTK_CONTAINER(item), hBox);
				break;
			
			// a check item
			case UMENU_CHECK:
				item = gtk_check_menu_item_new_with_mnemonic(tr(menulist[i].title));
				gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), menulist[i].selected);
				break;
			
			// a radio item
			case UMENU_RADIO:
				item = gtk_radio_menu_item_new_with_mnemonic(radioGroups[menulist[i].groupID], tr(menulist[i].title));
				gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), menulist[i].selected);

				if (!radioGroups[menulist[i].groupID])
					radioGroups[menulist[i].groupID] = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(item));
				else
					gtk_radio_menu_item_set_group(GTK_RADIO_MENU_ITEM(item), radioGroups[menulist[i].groupID]);					
				break;				
		}
		
		if (!item)
			continue;
		
		gtk_widget_set_sensitive(item, menulist[i].enabled);
		
		// add the submenu 
		if (menulist[i].subMenu)
			gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), GTK_WIDGET(menulist[i].subMenu));
				
		// connect the ID and the function
		g_object_set_data(G_OBJECT(item), "ICQmenuMENU", (gpointer)menuID);		
		g_object_set_data(G_OBJECT(item), "ICQmenuID", (gpointer)menulist[i].id);
				
		g_signal_connect(G_OBJECT(item),"activate", cbfunc, data);
		
		
		if (menulist[i].target && menulist[i].type != UMENU_HEADLINE)
			*menulist[i].target = item;
		else
		if (menulist[i].target && menulist[i].type == UMENU_HEADLINE)
			*menulist[i].target = title;
			
			
		gtk_widget_show_all(item);
		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);				
	}
	
	return GTK_MENU(menu);
}



void u_editMenuEntry(GtkWidget* menuEntry, gchar* newImage, gchar* newText)
{
	GList		*children;
	GtkWidget	*hbox,
				*img,
				*label;
	
	// get the hbox
	children = gtk_container_get_children(GTK_CONTAINER(menuEntry));
	hbox = (GtkWidget*)children->data;
	g_list_free(children);
	
	// get the widgets
	children = gtk_container_get_children(GTK_CONTAINER(hbox));
	img = (GtkWidget*)children->data;
	
	children = children->next;
	label = (GtkWidget*)children->data;
	
	g_list_free(children);
	
	if (newImage)
		gtk_image_set_from_stock(GTK_IMAGE(img), newImage, GTK_ICON_SIZE_MENU);
	
	if (newText)
	{
		gtk_label_set_label(GTK_LABEL(label), newText);
		gtk_label_set_use_underline(GTK_LABEL(label), TRUE);
	}
}



GdkPixbuf* u_getPixbufFromStock(gchar* stockName, GtkIconSize size)
{
	GtkWidget	*dummy;
	GdkPixbuf	*toReturn;
	
	dummy = gtk_toolbar_new();
	toReturn = gtk_widget_render_icon(dummy, stockName, size, NULL);
	gtk_widget_destroy(dummy);
	
	return toReturn;
}



GList* u_getUNumbersFromString(gchar* str)
{
	GList	*toReturn;
	gchar	*strEnd;
	guint	num;
	
	
	toReturn = NULL;
	while((strEnd = strchr(str, ',')))
	{
		num = strtoul(str, (char**)NULL, 10);
		toReturn = g_list_append(toReturn, GUINT_TO_POINTER(num));
		
		str = strEnd+1;
	}
	
	if (strlen(str))
	{
		num = strtoul(str, (char**)NULL, 10);
		toReturn = g_list_append(toReturn, GUINT_TO_POINTER(num));
	}
	return toReturn;
}


gchar* u_getStringFromNumbers(GList* list)
{
	GString	*toReturn;
	
	toReturn = g_string_new("");
	
	for (;list; list = list->next)
		g_string_append_printf(toReturn, "%d, ", GPOINTER_TO_UINT(list->data));
	
	if (*toReturn->str)
		toReturn = g_string_set_size(toReturn, strlen(toReturn->str)-2);
		
	return g_string_free(toReturn, FALSE);
}



void u_stripStringByNeedle(gchar* str, gchar* needle)
{
	gchar* pos;
	
	if ((pos = g_strstr_len(str+strlen(str)-3, 3, needle)))
		str[pos-str] = 0;		
}


void u_appendToString(gchar** str, gchar* toAppend)
{
	gchar	*tmp;
	
	tmp = g_strdup_printf("%s%s", *str, toAppend);
	g_free(*str);
	*str = tmp;
}



// X11 functions

borderInfo*	u_getDisplayBorders()
{
	
	Display			  *dpy;
	Window            root,
					  *children;	
	Window			  rootRet, parentRet;
	unsigned int	  noc, 
					  i,
					  result;
	int				  format;
	unsigned long	  n,
					  left;
	XWindowAttributes att;
	Atom			  dockAtom, 
					  typeAtom, 
					  strutAtom, 
					  strutPartialAtom,
					  actual,
					  prop;
	
	unsigned char	  *data, 
					  *data2;
	borderInfo 		  borders,
					  *toReturn;
	
	
	toReturn = (borderInfo*)malloc(sizeof(borderInfo));
	memset(toReturn, 0, sizeof(borderInfo));


	dpy = XOpenDisplay(NULL);
	
	dockAtom 		 = XInternAtom(dpy, "_NET_WM_WINDOW_TYPE_DOCK", False);
	typeAtom 		 = XInternAtom(dpy, "_NET_WM_WINDOW_TYPE", False);
	strutAtom 		 = XInternAtom(dpy, "_NET_WM_STRUT", False);
	strutPartialAtom = XInternAtom(dpy, "_NET_WM_STRUT_PARTIAL", False);

	root = RootWindow(dpy,DefaultScreen(dpy));
	
	
	// get all children of the root window
	XQueryTree(dpy, root, &rootRet, &parentRet, &children, &noc); 
	
	// get information about all the windows
	for (i=0;i<noc;i++)
	{
		// find out if we have a docked window
		result = XGetWindowProperty(dpy, children[i], typeAtom, 0L, 1L, False, XA_ATOM, &actual, &format, &n, &left, &data);

		if (result != Success || data == None)
			continue;

		memcpy(&prop, data, sizeof(Atom));
		if (prop == dockAtom)
		{
			// get the strut property
			result = XGetWindowProperty(dpy, children[i], strutPartialAtom, 0L, sizeof(long)*4, False, XA_CARDINAL, &actual, &format, &n, &left, &data2);

			// no success? try to get the older strut
			if (result != Success || data2 == None || actual == None)
				result = XGetWindowProperty(dpy, children[i], strutAtom, 0L, sizeof(long)*4, False, XA_CARDINAL, &actual, &format, &n, &left, &data2);
				
			if (result == Success && data2 != None && actual != None)
			{
				memcpy(&borders, data2, sizeof(long)*4);
				XGetWindowAttributes(dpy, children[i], &att);
				toReturn->left	 += borders.left;
				toReturn->right	 += borders.right;
				toReturn->top	 += borders.top;
				toReturn->bottom += borders.bottom;
					
				XFree(data2);
			}
		}
			
		XFree(data);
	}
	
	return toReturn; 
}


// FIXME:
// 1. make this work for other window managers
// 2. install a listener that checks for changes of the size of panel windows
//    and adjust our size properly.



gboolean cb_setWnd(GtkWidget	*wnd)
{
	// after resizing work was done by u_dockWindow (which calls this function)
	// here comes the interesting part -> allocate a dock area on the screen

	Display			*XDisplay;
	Window			XWindow;
	gulong			struts[12] = {0, };
	GdkRectangle	frameBorder;
	guint			dir;
	Atom			strutAtom;
	
	// get the docking direction (that was placed into wnd from u_dockWindow)
	dir = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(wnd), "icqnd-border-dir"));

	// get the window size incl. frame
	gdk_window_get_frame_extents(wnd->window, &frameBorder);
	
	XDisplay = GDK_WINDOW_XDISPLAY(wnd->window);
	XWindow = GDK_WINDOW_XWINDOW(wnd->window);
	
	
	// finally set the border info
	struts[dir] = frameBorder.width;
	
	// get the correct atom
	strutAtom = XInternAtom(XDisplay, "_NET_WM_STRUT_PARTIAL", False);
	
	if (!strutAtom)
		return TRUE;
	
	// and set it!
	XChangeProperty(XDisplay, XWindow, strutAtom, XA_CARDINAL, 32, 
					PropModeReplace, (guchar*)&struts, 12l);	
	
	
	return TRUE;	
}



gboolean u_dockWindow(GtkWidget* wnd, gchar dir)
{
	GdkRectangle	frameBorder;
	borderInfo		*displayBorders;
	gint			windowWidth,
					windowHeight,
					screenWidth,
					screenHeight;
	
	// check whether there is a window
	if (!wnd->window)
		return FALSE;
	
	// we do the docking in two steps
	// 1. set the window dimensions and position
	// 2. dock the window
	// this is neccessary in order to let windows be placed at screen bordered
	// regions. Normally our window manager wouldn't allow this. this is nothing
	// but a bad hack and only works with metacity.
	
	
	// get the current border info (space that is already reserved by panels ...)
	displayBorders = u_getDisplayBorders();
	
	// get window and screen dimensions
	gtk_window_get_size(GTK_WINDOW(wnd), &windowWidth, &windowHeight);
	gdk_window_get_frame_extents(wnd->window, &frameBorder);
	
	screenWidth = gdk_screen_get_width(gdk_screen_get_default());
	screenHeight = gdk_screen_get_height(gdk_screen_get_default());

	// now resize and position the window
	if (dir == STRUT_LEFT)
		gtk_window_move(GTK_WINDOW(wnd), displayBorders->left, displayBorders->top);
	else
		gtk_window_move(GTK_WINDOW(wnd), screenWidth - frameBorder.width - displayBorders->right, displayBorders->top);
	
	gdk_window_resize(wnd->window, windowWidth, 
					  screenHeight - (displayBorders->bottom + displayBorders->top) - (frameBorder.height - windowHeight));



	// finally start the timeout that asks the Xserver to 	
	g_object_set_data(G_OBJECT(wnd), "icqnd-border-dir", GUINT_TO_POINTER(dir));
	g_timeout_add(200, (GSourceFunc)cb_setWnd, wnd);

	return TRUE;		
}




void u_getDisplaySize(GtkWidget* w, gint* width, gint* height)
{	
	*width = XDisplayWidth(GDK_WINDOW_XDISPLAY(w->window), gdk_x11_get_default_screen());
	*height = XDisplayHeight(GDK_WINDOW_XDISPLAY(w->window), gdk_x11_get_default_screen());
}



void u_insertTextInterpretedIntoView(gchar* toAppend, gchar* tagName, gboolean createMargin, gboolean interpretURIs, gboolean interpretSmileys, GtkTextIter* pos, GtkTextView* viewToInsert)
{	
	smileysNode *child,
				*lastMatching;
	GList	*search;
	guint offset, i, l, lastOffset;
	gint	linkEnd, linkStart, i1;
	settings	*config;
	iconManager	*icons;
	GString		*insertSlice;
	GdkColor	*col;
	GtkTextChildAnchor *anchor;
	GtkTextBuffer	*buf;
	GtkWidget		*img;
	GtkTextTag		*tag,
					*tagMargin,
					*tagURL,
					*tagURLNew;
	gchar			*link;
		
	insertSlice = g_string_new("");
	config = settings_getSettings();
	icons = i_getIcons();
	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(viewToInsert));
	
	if (tagName)
		tag = gtk_text_tag_table_lookup(gtk_text_buffer_get_tag_table(buf), tagName);
	else
		tag = NULL;
	
	tagURL = gtk_text_tag_table_lookup(gtk_text_buffer_get_tag_table(buf), "url");
	g_object_get(G_OBJECT(tagURL), "foreground-gdk", &col, NULL);

	
	if (createMargin)
		tagMargin = gtk_text_tag_table_lookup(gtk_text_buffer_get_tag_table(buf), "margin");
	else
		tagMargin = NULL;
	
	search = NULL;
	
	l=0;
	
	linkEnd = -1;
	linkStart = 0;
	
	// go through the string
	for (i=0; i<strlen(toAppend); i++)
	{	
		
		if (interpretURIs)
		{
			// look for URLs
			// --------------
		
			// www/ftp-URLS
			if (((toAppend[i] == 'w' || toAppend[i] == 'W') && 
				toAppend[i+1] == toAppend[i] && 
				toAppend[i+2] == toAppend[i] &&
				toAppend[i+3] == '.') ||
		
		  		 ((toAppend[i] == 'f' || toAppend[i] == 'F') &&
				(toAppend[i+1] == 't' || toAppend[i+1] == 'T') &&
				(toAppend[i+2] == 'p' || toAppend[i+2] == 'P'))		
			)
			{			
				for (i1=i+3; i1<(gint)strlen(toAppend); i1++)
					if (toAppend[i1] == ' ' || toAppend[i1] == '\n')
					{
						linkEnd = i1;
						break;
					}
				
				if (linkEnd == -1)
					linkEnd = strlen(toAppend)-1;
			
				linkStart = i;
			} else
		
			// xxxx://-URLS
			if (toAppend[i] == ':' && toAppend[i+1] == '/' && toAppend[i+2] == '/')
			{
				// go back to the beginning of the protocol identifier
				// use insertSlice and not toAppend here. Text which is in
				// insertslice wasn't shown yet. This is not perfect and will
				// for example misunderstand something like htt:)p://...
			
				linkStart = i;
			
			
				for(i1=strlen(insertSlice->str)-1; i1>=-1; i1--)
				{
					if (i1 == -1 ||
						insertSlice->str[i1] == ' ' || 
						insertSlice->str[i1] == '\n' ||
						insertSlice->str[i1] == '.' ||
						insertSlice->str[i1] == '!' ||
						insertSlice->str[i1] == '?' ||
						insertSlice->str[i1] == '-' ||
						insertSlice->str[i1] == ',')
					{
						linkStart = i - (strlen(insertSlice->str)-i1)+1;
						// cut the insert slice so things won't be shown again
						g_string_set_size(insertSlice, i1+1);
						break;
					}
				} 
				
				// find the end
				for (i1=i; i1<(gint)strlen(toAppend); i1++)
					if (toAppend[i1] == ' ' || toAppend[i1] == '\n')
					{
						linkEnd = i1;
						break;
					}
				
				if (linkEnd == -1)
					linkEnd = strlen(toAppend)-1;
			}
		
		
			if (linkEnd != -1)
			{
			
				// insert the old slice content
				// gtk_text_buffer_get_end_iter(buf, &iter);
				gtk_text_buffer_insert_with_tags(buf, pos, insertSlice->str, strlen(insertSlice->str), tag, tagMargin, NULL);
				g_string_set_size(insertSlice, 0);

				// create a new tag and add the appropriate link
				tagURLNew = gtk_text_tag_new(NULL);
				g_object_set(G_OBJECT(tagURLNew), "underline", PANGO_UNDERLINE_SINGLE,
											  NULL);
				gtk_text_tag_table_add(gtk_text_buffer_get_tag_table(buf), tagURLNew);
			
				link = (gchar*)g_malloc0((linkEnd - linkStart)*sizeof(gchar)+1);
				memcpy(link, toAppend + linkStart, (linkEnd - linkStart)*sizeof(gchar));
				g_object_set_data_full(G_OBJECT(tagURLNew), "ICQLink", link, g_free);
			
				// insert the link
				// gtk_text_buffer_get_end_iter(buf, &iter);
				gtk_text_buffer_insert_with_tags(buf, pos, toAppend + linkStart, linkEnd - linkStart, tagURLNew, tag, tagMargin, NULL);

				i+=(linkEnd - linkStart) - (i - linkStart);			
				linkEnd = -1;
			}
		}
	
		
		// look for smileys
		// ----------------
		if (interpretSmileys)
		{
			offset 		 = 0;
			lastMatching = NULL;
			lastOffset	 = 0;
			
			// begin at the root of the smileys tree
			if ( icons != NULL && icons -> smileyTree != NULL && icons -> smileyTree -> children != NULL ) {
				search = icons->smileyTree->children;
			} else {
				search = NULL;
			}
			
			while(search)
			{
				l++;
				child = (smileysNode*)search->data;
				
				// if a smiley character matches it or one of it's children could be 
				// the target
				if (toAppend[offset+i] == child->character && strlen(toAppend)>offset+i)
				{
					// is this already a smiley or just a node?
					if (child->smileysIndex != -1)
					{
						lastMatching = child;
						lastOffset = offset;
					}
						
					// if there are children go on searching
					if (child->children)
					{
						search = child->children;
						offset++;
						
						continue;
					}
				}
				
				search = search->next;
			}
				
			// if we have a match add the smiley, otherwise just add the character
			if (lastMatching)
			{
				gtk_text_buffer_insert_with_tags(buf, pos, insertSlice->str, strlen(insertSlice->str), tag, tagMargin, NULL);
					
				g_string_set_size(insertSlice, 0);
								
				img = gtk_image_new_from_pixbuf(icons->images.smileys[lastMatching->smileysIndex].icon);
				gtk_widget_show(img);
			
				anchor = gtk_text_buffer_create_child_anchor(buf, pos);
				gtk_text_view_add_child_at_anchor(GTK_TEXT_VIEW(viewToInsert), img, anchor);

				i+=lastOffset;
			} else
			g_string_append_c(insertSlice, toAppend[i]);
		} else
		g_string_append_c(insertSlice, toAppend[i]);		
	
	} 
	

	// insert the rest
	if (strlen(insertSlice->str)>0)
		gtk_text_buffer_insert_with_tags(buf, pos, insertSlice->str, strlen(insertSlice->str), tag, tagMargin, NULL);
	
	g_string_free(insertSlice, TRUE);
}



void u_checkInterpretedViewForURLCursor(gint x, gint y, GtkTextView* view, gchar** hoveredLink)
{
	GSList			*tags,
					*search;
	GtkTextBuffer	*buf;
	GtkTextIter		iter;
	gint			bX,
					bY;
	GtkTextTag		*curTag;
	gchar			*curLink;
	




 	gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW (view), 
                                          GTK_TEXT_WINDOW_WIDGET,
                                          x, y, &bX, &bY);
		
	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
	
	gtk_text_view_get_iter_at_location(GTK_TEXT_VIEW(view), &iter, bX, bY);
	
	
	tags = gtk_text_iter_get_tags(&iter);
	search = tags;
	while(search)
	{
		curTag = (GtkTextTag*)search->data;
		
		curLink = (gchar*)g_object_get_data(G_OBJECT(curTag), "ICQLink");
		
		if (curLink)
		{
			if (!*hoveredLink)
			{
				if (!u_cursorHover)
					u_cursorHover = gdk_cursor_new(GDK_HAND2);
				
				gdk_window_set_cursor(gtk_text_view_get_window(view, GTK_TEXT_WINDOW_TEXT), u_cursorHover);
				*hoveredLink = curLink;
			}
			
			break;
		}
		
		search = search->next;
	}
	
	if (!search && *hoveredLink != NULL)
	{
		if (!u_cursorNormal)
			u_cursorNormal = gdk_cursor_new(GDK_XTERM);
		gdk_window_set_cursor(gtk_text_view_get_window(view, GTK_TEXT_WINDOW_TEXT), u_cursorNormal);
		*hoveredLink = NULL;
	}
	
	g_slist_free(tags);
}
